﻿--[[
Name: StatLogic-1.0
Description: A Library for stat conversion, calculation and summarization.
Revision: $Revision: 30315 $
Author: Whitetooth@Cenarius (hotdogee@巴哈姆特baha)
LastUpdate: $Date: 2007-03-15 22:46:58 -0400 (Thu, 15 Mar 2007) $
Website:
Documentation:
SVN: $URL: http://svn.wowace.com/wowace/trunk/StatLogicLib/StatLogic-1.0/StatLogic-1.0.lua $
Dependencies: AceLibrary
License: LGPL v2.1
Design Aim: Indexing algorithm instead of calling strfind for every stat
]]

-- This library is still in early development

local MAJOR_VERSION = "StatLogic-1.0"
local MINOR_VERSION = tonumber(("$Revision: 30315 $"):sub(12, -3))

if not AceLibrary then error(MAJOR_VERSION.." requires AceLibrary") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end


-------------------------
-- Localization Tables --
-------------------------
local L
L = {
	["Locale"] = "enUS",
	["Dual Wield"] = "Dual Wield",
	-- By looking at the first ExcludeLen letters of a line we can exclude a lot of lines
	["ExcludeLen"] = 5,
	["Exclude"] = {
		[""] = true,
		[" \n"] = true,
		["Binds when equipped"] = true,
		["Binds when picked up"] = true,
		["Binds when used"] = true,
		["Red Socket"] = true,
		["Yellow Socket"] = true,
		["Blue Socket"] = true,
		["Meta Socket"] = true,
		["Quest Item"] = true,
		["This Item Begins a Quest"] = true,
		["Uniqu"] = true, -- Unique (20)
		["Requi"] = true, -- Requires Level xx
		["\nRequ"] = true, -- Requires Level xx
		["Class"] = true, -- Classes: xx
		["Races:"] = true, -- Races: xx (vendor mounts)
		["Use: "] = true, -- Use:
		["Chanc"] = true, -- Chance On Hit:
		-- Set Bonuses
		["Set: "] = true,
		["(2) S"] = true,
		["(2) S"] = true,
		["(3) S"] = true,
		["(4) S"] = true,
		["(5) S"] = true,
		["(6) S"] = true,
		["(7) S"] = true,
		["(8) S"] = true,
		-- Equip type
		["Projectile"] = true, -- Ice Threaded Arrow ID:19316
		[INVTYPE_AMMO] = true,
		[INVTYPE_HEAD] = true,
		[INVTYPE_NECK] = true,
		[INVTYPE_SHOULDER] = true,
		[INVTYPE_BODY] = true,
		[INVTYPE_CHEST] = true,
		[INVTYPE_ROBE] = true,
		[INVTYPE_WAIST] = true,
		[INVTYPE_LEGS] = true,
		[INVTYPE_FEET] = true,
		[INVTYPE_WRIST] = true,
		[INVTYPE_HAND] = true,
		[INVTYPE_FINGER] = true,
		[INVTYPE_TRINKET] = true,
		[INVTYPE_CLOAK] = true,
		[INVTYPE_WEAPON] = true,
		[INVTYPE_SHIELD] = true,
		[INVTYPE_2HWEAPON] = true,
		[INVTYPE_WEAPONMAINHAND] = true,
		[INVTYPE_WEAPONOFFHAND] = true,
		[INVTYPE_HOLDABLE] = true,
		[INVTYPE_RANGED] = true,
		[INVTYPE_THROWN] = true,
		[INVTYPE_RANGEDRIGHT] = true,
		[INVTYPE_RELIC] = true,
		[INVTYPE_TABARD] = true,
		[INVTYPE_BAG] = true,
	},
	-- Special cases that need to be dealt with before base scan
	["PreScanPatterns"] = {
		--["^Equip: Increases attack power by (%d+) in Cat"] = "FERAL_ATTACKPOWER",
		--["^Equip: Increases attack power by (%d+) when fighting Undead"] = "ATTACKPOWER_UNDEAD", -- Seal of the Dawn ID:13029
		["^(%d+) Block$"] = "BLOCK_VALUE",
		["^(%d+) Armor$"] = "ARMOR",
		["Reinforced %(%+(%d+) Armor%)"] = "ARMOR_BUFF",
		["Mana Regen (%d+) per 5 sec%.$"] = "MANA_REG",
		["^%+?%d+ %- (%d+) .-Damage$"] = "MAX_DAMAGE",
		["^%(([%d%.]+) damage per second%)$"] = "DPS",
		-- Exclude
		["^(%d+) Slot"] = false, -- Set Name (0/9)
		["^[%a '%-]+%((%d+)/%d+%)$"] = false, -- Set Name (0/9)
		["|cff808080"] = false, -- Gray text "  |cff808080Requires at least 2 Yellow gems|r\n  |cff808080Requires at least 1 Red gem|r"
		-- Procs
		["[Cc]hance"] = false, -- [Mark of Defiance] ID:27924 -- [Staff of the Qiraji Prophets] ID:21128
		["[Ss]ometimes"] = false, -- [Darkmoon Card: Heroism] ID:19287
		["[Ww]hen struck in combat"] = false, -- [Essence of the Pure Flame] ID: 18815
	},
	-- Whole Text Lookup
	-- Mainly used for enchants that doesn't have numbers in the text
	["WholeTextLookup"] = {
		["Mithril Spurs"] = {["MOUNT_SPEED%"] = 4}, -- Mithril Spurs
		["Minor Mount Speed Increase"] = {["MOUNT_SPEED%"] = 2}, -- Enchant Gloves - Riding Skill
		["Equip: Run speed increased slightly."] = {["RUN_SPEED%"] = 8}, -- [Highlander's Plate Greaves] ID: 20048
		["Run speed increased slightly"] = {["RUN_SPEED%"] = 8}, --
		["Minor Speed Increase"] = {["RUN_SPEED%"] = 8}, -- Enchant Boots - Minor Speed
		["Equip: Allows underwater breathing."] = false, -- [Band of Icy Depths] ID: 21526
		["Allows underwater breathing"] = false, --
		["Equip: Immune to Disarm."] = false, -- [Stronghold Gauntlets] ID: 12639
		["Immune to Disarm"] = false, --
		["Crusader"] = false, -- Enchant Crusader
		["Lifestealing"] = false, -- Enchant Crusader
	},
	--[[
	textTable = {
		"Spell Damage +6 and Spell Hit Rating +5",
		"+3 Stamina, +4 Critical Strike Rating",
		"+26 Healing Spells & 2% Reduced Threat",
		"+3 Stamina/+4 Critical Strike Rating",
		"Socket Bonus: 2 mana per 5 sec.",
		"Equip: Increases damage and healing done by magical spells and effects by up to 150.",
		"Equip: Increases the spell critical strike rating of all party members within 30 yards by 28.",
		"Equip: Increases damage and healing done by magical spells and effects of all party members within 30 yards by up to 33.",
		"Equip: Increases healing done by magical spells and effects of all party members within 30 yards by up to 62.",
		"Equip: Increases your spell damage by up to 120 and your healing by up to 300.",
		"Equip: Restores 11 mana per 5 seconds to all party members within 30 yards.",
		"Equip: Increases healing done by spells and effects by up to 300.",
		"Equip: Increases attack power by 420 in Cat, Bear, Dire Bear, and Moonkin forms only.",
		-- "+10 Defense Rating/+10 Stamina/+15 Block Value": ZG Enchant
		-- "+26 Attack Power and +14 Critical Strike Rating": Swift Windfire Diamond ID:28556
		-- "+26 Healing Spells & 2% Reduced Threat": Bracing Earthstorm Diamond ID:25897
		-- "+6 Spell Damage, +5 Spell Crit Rating": Potent Ornate Topaz ID: 28123
		----
		-- "Critical Rating +6 and Dodge Rating +5": Assassin's Fire Opal ID:30565
		-- "Healing +11 and 2 mana per 5 sec.": Royal Tanzanite ID: 30603
	}
	--]]
	-- Single Plus Stat Check
	-- depending on locale, it may be
	-- +19 Stamina = "^%+(%d+) ([%a ]+%a)$"
	-- Stamina +19 = "^([%a ]+%a) %+(%d+)$"
	-- +19 耐力 = "^%+(%d+) (.-)$"
	--["SinglePlusStatCheck"] = "^%+(%d+) ([%a ]+%a)$",
	["SinglePlusStatCheck"] = "^%+(%d+) (.-)$",
	-- Single Equip Stat Check
	-- stat1, value, stat2 = strfind
	-- stat = stat1..stat2
	-- "^Equip: (.-) by u?p? ?t?o? ?(%d+) ?(.-)%.$"
	["SingleEquipStatCheck"] = "^Equip: (.-) by u?p? ?t?o? ?(%d+) ?(.-)%.$",
	-- Initial "+" scan
	-- Replace " and " with "@" for "+22 xxx xx and +22 xxx xxxx" type strings
	[" and "] = " and ",
	["%+(%d+) ([%a ]+%a) ?"] = "%+(%d+) ([%a ]+%a) ?",
	-- DeepScan
	["Equip: "] = "Equip: ",
	["DeepScanSeparators"] = {
		" and ", -- "Critical Rating +6 and Dodge Rating +5": Assassin's Fire Opal ID:30565
		"/", -- "+10 Defense Rating/+10 Stamina/+15 Block Value": ZG Enchant
		" & ", -- "+26 Healing Spells & 2% Reduced Threat": Bracing Earthstorm Diamond ID:25897
		", ", -- "+6 Spell Damage, +5 Spell Crit Rating": Potent Ornate Topaz ID: 28123
		"%. ", -- "Equip: Increases attack power by 81 when fighting Undead. It also allows the acquisition of Scourgestones on behalf of the Argent Dawn.": Seal of the Dawn
	},
	--[[ OLD
	["DeepScanPatterns"] = {
		"(%a[%a ]+%a) ?%d* ?%a* by u?p? ?t?o? ?(%d+) ?a?n?d? ?", -- xxx xxx by 22 (scan first)
		"(%a[%a ]+) %+(%d+) ?a?n?d? ?", -- xxx xxx +22 (scan 2ed)
		"(%d+) ([%a ]+%a) ?a?n?d? ?", -- 22 xxx xxx (scan last)
	},
	--]]
	["DeepScanPatterns"] = {
		"^(.-) by u?p? ?t?o? ?(%d+) ?(.-)$", -- "xxx by up to 22 xxx" (scan first)
		"^(.-) ?%+(%d+) ?(.-)$", -- "xxx xxx +22" or "+22 xxx xxx" or "xxx +22 xxx" (scan 2ed)
		"^(.-) ?([%d%.]+) ?(.-)$", -- 22.22 xxx xxx (scan last)
	},
	["StatIDLookup"] = {

		["Increases your effective stealth level"] = {"STEALTH_LEVEL"}, -- [Nightscape Boots] ID: 8197
		["Weapon Damage"] = {"MELEE_DMG"}, -- Enchant
		["Increases mount speed%"] = {"MOUNT_SPEED%"}, -- [Highlander's Plate Greaves] ID: 20048

		["All Stats"] = {"STR", "AGI", "STA", "INT", "SPI",},
		["Strength"] = {"STR",},
		["Agility"] = {"AGI",},
		["Stamina"] = {"STA",},
		["Intellect"] = {"INT",},
		["Spirit"] = {"SPI",},

		["Arcane Resistance"] = {"ARCANE_RES",},
		["Fire Resistance"] = {"FIRE_RES",},
		["Nature Resistance"] = {"NATURE_RES",},
		["Frost Resistance"] = {"FROST_RES",},
		["Shadow Resistance"] = {"SHADOW_RES",},
		["Shadow resistance"] = {"SHADOW_RES",}, -- Demons Blood ID: 10779
		["All Resistances"] = {"ARCANE_RES", "FIRE_RES", "FROST_RES", "NATURE_RES", "SHADOW_RES",},
		["Resist All"] = {"ARCANE_RES", "FIRE_RES", "FROST_RES", "NATURE_RES", "SHADOW_RES",},

		["Fishing"] = {"FISHING",}, -- Fishing enchant ID:846
		["Fishing Skill"] = {"FISHING",}, -- Fishing lure
		["Increased Fishing"] = {"FISHING",}, -- Equip: Increased Fishing +20.
		["Mining"] = {"MINING",}, -- Mining enchant ID:844
		["Herbalism"] = {"HERBALISM",}, -- Heabalism enchant ID:845
		["Skinning"] = {"SKINNING",}, -- Skinning enchant ID:865

		["Armor"] = {"ARMOR",},
		["Defense"] = {"DEFENSE",},
		["Increased Defense"] = {"DEFENSE",},
		["Block"] = {"BLOCK_VALUE",}, -- +22 Block Value
		["Block Value"] = {"BLOCK_VALUE",}, -- +22 Block Value
		["Increases the block value of your shield"] = {"BLOCK_VALUE",},

		["Health"] = {"HEALTH",},
		["HP"] = {"HEALTH",},
		["Mana"] = {"MANA",},

		["Attack Power"] = {"ATTACKPOWER",},
		["Increases attack power"] = {"ATTACKPOWER",},
		["Attack Power when fighting Undead"] = {"ATTACKPOWER_UNDEAD",},
		["Increases attack powerwhen fighting Undead"] = {"ATTACKPOWER_UNDEAD",},
		["Increases attack powerwhen fighting Undead.  It also allows the acquisition of Scourgestones on behalf of the Argent Dawn"] = {"ATTACKPOWER_UNDEAD",},
		["Increases attack powerwhen fighting Demons"] = {"ATTACKPOWER_DEMON",},
		["Attack Power in Cat, Bear, and Dire Bear forms only"] = {"FERAL_ATTACKPOWER",},
		["Increases attack powerin Cat, Bear, Dire Bear, and Moonkin forms only"] = {"FERAL_ATTACKPOWER",},
		["Ranged Attack Power"] = {"RANGED_ATTACKPOWER",},
		["Increases ranged attack power"] = {"RANGED_ATTACKPOWER",}, -- [High Warlord's Crossbow] ID: 18837

		["Health Regen"] = {"MANA_REG",},
		["Health per"] = {"HEALTH_REG",},
		["health per"] = {"HEALTH_REG",}, -- Frostwolf Insignia Rank 6 ID:17909
		["Health every"] = {"MANA_REG",},
		["health every"] = {"HEALTH_REG",}, -- [Resurgence Rod] ID:17743
		["your normal health regeneration"] = {"HEALTH_REG",}, -- Demons Blood ID: 10779
		["Restoreshealth per 5 sec"] = {"HEALTH_REG",}, -- [Onyxia Blood Talisman] ID: 18406
		["Restoreshealth every 5 sec"] = {"HEALTH_REG",}, -- [Resurgence Rod] ID:17743
		["Mana Regen"] = {"MANA_REG",},
		["Mana per"] = {"MANA_REG",},
		["mana per"] = {"MANA_REG",}, -- Resurgence Rod ID:17743 Most common
		["Mana every"] = {"MANA_REG",},
		["mana every"] = {"MANA_REG",},
		["Mana every 5 seconds"] = {"MANA_REG",}, -- [Royal Nightseye] ID: 24057
		["Mana per 5 Seconds"] = {"MANA_REG",}, -- [Royal Shadow Draenite] ID: 23109
		["Restoresmana per 5 sec"] = {"MANA_REG",}, -- [Resurgence Rod] ID:17743

		["Spell Penetration"] = {"SPELLPEN",},
		["Increases your spell penetration"] = {"SPELLPEN",},
		["Healing and Spell Damage"] = {"DMG", "HEAL",},
		["Damage and Healing Spells"] = {"DMG", "HEAL",},
		["Spell Damage and Healing"] = {"DMG", "HEAL",},
		["Spell Damage"] = {"DMG", "HEAL",},
		["Increases damage and healing done by magical spells and effects"] = {"DMG", "HEAL"},
		["Increases damage and healing done by magical spells and effects of all party members within 30 yards"] = {"DMG", "HEAL"}, -- Atiesh
		["Spell Damage and Healing"] = {"DMG", "HEAL",},
		--StatLogic:GetSum("item:22630")
		["Damage"] = {"DMG",},
		["Increases your spell damage"] = {"DMG",}, -- Atiesh ID:22630, 22631, 22632, 22589
		["Holy Damage"] = {"HOLY_DMG",},
		["Arcane Damage"] = {"ARCANE_DMG",},
		["Fire Damage"] = {"FIRE_DMG",},
		["Nature Damage"] = {"NATURE_DMG",},
		["Frost Damage"] = {"FROST_DMG",},
		["Shadow Damage"] = {"SHADOW_DMG",},
		["Holy Spell Damage"] = {"HOLY_DMG",},
		["Arcane Spell Damage"] = {"ARCANE_DMG",},
		["Fire Spell Damage"] = {"FIRE_DMG",},
		["Nature Spell Damage"] = {"NATURE_DMG",},
		["Frost Spell Damage"] = {"FROST_DMG",}, -- Acrobatic Staff of Frozen Wrath ID:3185:0:0:0:0:0:1957
		["Shadow Spell Damage"] = {"SHADOW_DMG",},
		["Increases damage done by Arcane spells and effects"] = {"ARCANE_DMG",},
		["Increases damage done by Fire spells and effects"] = {"FIRE_DMG",},
		["Increases damage done by Frost spells and effects"] = {"FROST_DMG",}, -- Frozen Shadoweave Vest ID:21871
		["Increases damage done by Holy spells and effects"] = {"HOLY_DMG",},
		["Increases damage done by Nature spells and effects"] = {"NATURE_DMG",},
		["Increases damage done by Shadow spells and effects"] = {"SHADOW_DMG",}, -- Frozen Shadoweave Vest ID:21871
		["Increases damage done to Undead by magical spells and effects"] = {"DMG_UNDEAD"},
		["Increases damage done to Undead by magical spells and effects.  It also allows the acquisition of Scourgestones on behalf of the Argent Dawn"] = {"DMG_UNDEAD"},

		["Healing Spells"] = {"HEAL",},
		["Increases Healing"] = {"HEAL",},
		["Healing"] = {"HEAL",},
		["healing Spells"] = {"HEAL",},
		["Healing Spells"] = {"HEAL",}, -- [Royal Nightseye] ID: 24057
		["Increases healing done by spells and effects"] = {"HEAL",},
		["Increases healing done by magical spells and effects of all party members within 30 yards"] = {"HEAL",}, -- Atiesh
		["your healing"] = {"HEAL",}, -- Atiesh

		["damage per second"] = {"DPS",},
		["Addsdamage per second"] = {"DPS",}, -- [Thorium Shells] ID: 15977

		--[[ Rating ID
		CR_WEAPON_SKILL = 1;
		CR_DEFENSE_SKILL = 2;
		CR_DODGE = 3;
		CR_PARRY = 4;
		CR_BLOCK = 5;
		CR_HIT_MELEE = 6;
		CR_HIT_RANGED = 7;
		CR_HIT_SPELL = 8;
		CR_CRIT_MELEE = 9;
		CR_CRIT_RANGED = 10;
		CR_CRIT_SPELL = 11;
		CR_HIT_TAKEN_MELEE = 12;
		CR_HIT_TAKEN_RANGED = 13;
		CR_HIT_TAKEN_SPELL = 14;
		CR_CRIT_TAKEN_MELEE = 15;
		CR_CRIT_TAKEN_RANGED = 16;
		CR_CRIT_TAKEN_SPELL = 17;
		CR_HASTE_MELEE = 18;
		CR_HASTE_RANGED = 19;
		CR_HASTE_SPELL = 20;
		CR_WEAPON_SKILL_MAINHAND = 21;
		CR_WEAPON_SKILL_OFFHAND = 22;
		CR_WEAPON_SKILL_RANGED = 23;
		--]]
		["Defense Rating"] = {"CR_DEFENSE",},
		["Increases defense rating"] = {"CR_DEFENSE",},
		["Dodge Rating"] = {"CR_DODGE",},
		["Increases your dodge rating"] = {"CR_DODGE",},
		["Parry Rating"] = {"CR_PARRY",},
		["Increases your parry rating"] = {"CR_PARRY",},
		["Block Rating"] = {"CR_BLOCK",},
		["Increases your block rating"] = {"CR_BLOCK",},
		["Increases your shield block rating"] = {"CR_BLOCK",},

		["Hit Rating"] = {"CR_HIT_MELEE",},
		["Improves hit rating"] = {"CR_HIT_MELEE",}, -- ITEM_MOD_HIT_RATING
		["Improves melee hit rating"] = {"CR_HIT_MELEE",}, -- ITEM_MOD_HIT_MELEE_RATING
		["Increases your hit rating"] = {"CR_HIT_MELEE",},
		["Spell Hit Rating"] = {"CR_HIT_SPELL",},
		["Improves spell hit rating"] = {"CR_HIT_SPELL",}, -- ITEM_MOD_HIT_SPELL_RATING
		["Increases your spell hit rating"] = {"CR_HIT_SPELL",},
		["Ranged Hit Rating"] = {"CR_HIT_RANGED",},
		["Improves ranged hit rating"] = {"CR_HIT_RANGED",}, -- ITEM_MOD_HIT_RANGED_RATING
		["Increases your ranged hit rating"] = {"CR_HIT_RANGED",},

		["Crit Rating"] = {"CR_CRIT_MELEE",},
		["Critical Rating"] = {"CR_CRIT_MELEE",},
		["Critical Strike Rating"] = {"CR_CRIT_MELEE",},
		["Increases your critical hit rating"] = {"CR_CRIT_MELEE",},
		["Increases your critical strike rating"] = {"CR_CRIT_MELEE",},
		["Improves critical strike rating"] = {"CR_CRIT_MELEE",},
		["Spell Critical Strike Rating"] = {"CR_CRIT_SPELL",},
		["Spell Critical strike rating"] = {"CR_CRIT_SPELL",},
		["Spell Critical Rating"] = {"CR_CRIT_SPELL",},
		["Spell Crit Rating"] = {"CR_CRIT_SPELL",},
		["Increases your spell critical strike rating"] = {"CR_CRIT_SPELL",},
		["Increases the spell critical strike rating of all party members within 30 yards"] = {"CR_CRIT_SPELL",},
		["Improves spell critical strike rating"] = {"CR_CRIT_SPELL",},
		["Increases your ranged critical strike rating"] = {"CR_CRIT_RANGED",}, -- Fletcher's Gloves ID:7348

		["Improves hit avoidance rating"] = {"CR_HIT_TAKEN_MELEE"}, -- ITEM_MOD_HIT_TAKEN_RATING
		["Improves melee hit avoidance rating"] = {"CR_HIT_TAKEN_MELEE"}, -- ITEM_MOD_HIT_TAKEN_MELEE_RATING
		["Improves ranged hit avoidance rating"] = {"CR_HIT_TAKEN_RANGED"}, -- ITEM_MOD_HIT_TAKEN_RANGED_RATING
		["Improves spell hit avoidance rating"] = {"CR_HIT_TAKEN_SPELL"}, -- ITEM_MOD_HIT_TAKEN_SPELL_RATING
		["Resilience"] = {"CR_RESILIENCE",},
		["Resilience Rating"] = {"CR_RESILIENCE",},
		["Improves your resilience rating"] = {"CR_RESILIENCE",},
		["Improves critical avoidance rating"] = {"CR_CRIT_TAKEN_MELEE",},
		["Improves melee critical avoidance rating"] = {"CR_CRIT_TAKEN_MELEE",},
		["Improves ranged critical avoidance rating"] = {"CR_CRIT_TAKEN_RANGED",},
		["Improves spell critical avoidance rating"] = {"CR_CRIT_TAKEN_SPELL",},

		["Haste Rating"] = {"CR_HASTE_MELEE"},
		["Spell Haste Rating"] = {"CR_HASTE_SPELL"},
		["Ranged Haste Rating"] = {"CR_HASTE_RANGED"},
		["Improves haste rating"] = {"CR_HASTE_MELEE"},
		["Improves melee haste rating"] = {"CR_HASTE_MELEE"},
		["Improves ranged haste rating"] = {"CR_HASTE_SPELL"},
		["Improves spell haste rating"] = {"CR_HASTE_RANGED"},

		["Increases dagger skill rating"] = {"CR_WEAPON_DAGGER"},
		["Increases sword skill rating"] = {"CR_WEAPON_SWORD"},
		["Increases Two-Handed Swords skill rating"] = {"CR_WEAPON_SWORD_2H"},
		["Increases axe skill rating"] = {"CR_WEAPON_AXE"},
		["Two-Handed Axe Skill Rating"] = {"CR_WEAPON_AXE_2H"},
		["Increases two-handed axes skill rating"] = {"CR_WEAPON_AXE_2H"},
		["Increases mace skill rating"] = {"CR_WEAPON_MACE"},
		["Increases two-handed maces skill rating"] = {"CR_WEAPON_MACE_2H"},
		["Increases gun skill rating"] = {"CR_WEAPON_GUN"},
		["Increases Crossbow skill rating"] = {"CR_WEAPON_CROSSBOW"},
		["Increases Bow skill rating"] = {"CR_WEAPON_BOW"},
		["Increases feral combat skill rating"] = {"CR_WEAPON_FERAL"},
		["Increases fist weapons skill rating"] = {"CR_WEAPON_FIST"}, -- Demonblood Eviscerator
		["Increases unarmed skill rating"] = {"CR_WEAPON_FIST"}, -- Demonblood Eviscerator ID:27533
		-- Exclude
		["sec"] = false,
		["to"] = false,
		["Slot Bag"] = false,
		["Slot Quiver"] = false,
		["Slot Ammo Pouch"] = false,
		["Increases ranged attack speed"] = false, -- AV quiver
	},
	-- StatID are compatible with ItemBonusLib and BonusScanner
	["StatIDToName"] = {
		["STEALTH_LEVEL"] = {"Stealth Level", "Stealth"},
		["MELEE_DMG"] = {"Melee Weapon Damage", "Weapon Damage"},
		["MOUNT_SPEED%"] = {"Mount Speed(%)", "Mount Speed(%)"},
		["RUN_SPEED%"] = {"Run Speed(%)", "Run Speed(%)"},

		["STR"] = {SPELL_STAT1_NAME, "Str"},
		["AGI"] = {SPELL_STAT2_NAME, "Agi"},
		["STA"] = {SPELL_STAT3_NAME, "Sta"},
		["INT"] = {SPELL_STAT4_NAME, "Int"},
		["SPI"] = {SPELL_STAT5_NAME, "Spi"},
		["ARMOR"] = {ARMOR.." from items", "ARMOR(Items)"},
		["ARMOR_BUFF"] = {ARMOR.." from bonus", "ARMOR(Bonus)"},

		["FIRE_RES"] = {RESISTANCE2_NAME, "FR"},
		["NATURE_RES"] = {RESISTANCE3_NAME, "NR"},
		["FROST_RES"] = {RESISTANCE4_NAME, "FrR"},
		["SHADOW_RES"] = {RESISTANCE5_NAME, "SR"},
		["ARCANE_RES"] = {RESISTANCE6_NAME, "AR"},

		["FISHING"] = {"Fishing", "Fishing"},
		["MINING"] = {"Mining", "Mining"},
		["HERBALISM"] = {"Herbalism", "Herbalism"},
		["SKINNING"] = {"Skinning", "Skinning"},

		["BLOCK_VALUE"] = {"Block Value", "Block Value"},

		["ATTACKPOWER"] = {ATTACK_POWER_TOOLTIP, "AP"},
		["RANGED_ATTACKPOWER"] = {RANGED_ATTACK_POWER, "RAP"},
		["FERAL_ATTACKPOWER"] = {ATTACK_POWER_TOOLTIP.." (Feral)", "AP(Feral)"},
		["ATTACKPOWER_UNDEAD"] = {ATTACK_POWER_TOOLTIP.." (Undead)", "AP(Undead)"},
		["ATTACKPOWER_DEMON"] = {ATTACK_POWER_TOOLTIP.." (Demon)", "AP(Demon)"},

		["HEAL"] = {"Healing", "Heal"},

		["DMG"] = {PLAYERSTAT_SPELL_COMBAT.." "..DAMAGE, PLAYERSTAT_SPELL_COMBAT.." Dmg"},
		["DMG_UNDEAD"] = {PLAYERSTAT_SPELL_COMBAT.." "..DAMAGE.." (Undead)", PLAYERSTAT_SPELL_COMBAT.." Dmg".." (Undead)"},
		["HOLY_DMG"] = {SPELL_SCHOOL1_CAP.." "..DAMAGE, SPELL_SCHOOL1_CAP.." Dmg"},
		["FIRE_DMG"] = {SPELL_SCHOOL2_CAP.." "..DAMAGE, SPELL_SCHOOL2_CAP.." Dmg"},
		["NATURE_DMG"] = {SPELL_SCHOOL3_CAP.." "..DAMAGE, SPELL_SCHOOL3_CAP.." Dmg"},
		["FROST_DMG"] = {SPELL_SCHOOL4_CAP.." "..DAMAGE, SPELL_SCHOOL4_CAP.." Dmg"},
		["SHADOW_DMG"] = {SPELL_SCHOOL5_CAP.." "..DAMAGE, SPELL_SCHOOL5_CAP.." Dmg"},
		["ARCANE_DMG"] = {SPELL_SCHOOL6_CAP.." "..DAMAGE, SPELL_SCHOOL6_CAP.." Dmg"},

		["SPELLPEN"] = {PLAYERSTAT_SPELL_COMBAT.." "..SPELL_PENETRATION, SPELL_PENETRATION},

		["HEALTH_REG"] = {HEALTH.." Regen", "HP5"},
		["MANA_REG"] = {MANA.." Regen", "MP5"},
		["HEALTH"] = {HEALTH, HP},
		["MANA"] = {MANA, MP},

		["MAX_DAMAGE"] = {"Max Damage", "Max Dmg"},
		["DPS"] = {"Damage Per Second", "DPS"},

		["CR_DEFENSE"] = {COMBAT_RATING_NAME2, COMBAT_RATING_NAME2}, -- COMBAT_RATING_NAME2 = "Defense Rating"
		["CR_DODGE"] = {COMBAT_RATING_NAME3, COMBAT_RATING_NAME3}, -- COMBAT_RATING_NAME3 = "Dodge Rating"
		["CR_PARRY"] = {COMBAT_RATING_NAME4, COMBAT_RATING_NAME4}, -- COMBAT_RATING_NAME4 = "Parry Rating"
		["CR_BLOCK"] = {COMBAT_RATING_NAME5, COMBAT_RATING_NAME5}, -- COMBAT_RATING_NAME5 = "Block Rating"
		["CR_HIT_MELEE"] = {COMBAT_RATING_NAME6, COMBAT_RATING_NAME6}, -- COMBAT_RATING_NAME6 = "Hit Rating"
		["CR_HIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME6, PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME6}, -- PLAYERSTAT_RANGED_COMBAT = "Ranged"
		["CR_HIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME6, PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME6}, -- PLAYERSTAT_SPELL_COMBAT = "Spell"
		["CR_HIT_TAKEN_MELEE"] = {"Hit Avoidance "..RATING, "Hit Avoidance "..RATING},
		["CR_HIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance "..RATING, PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance "..RATING},
		["CR_HIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance "..RATING, PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance "..RATING},
		["CR_CRIT_MELEE"] = {COMBAT_RATING_NAME9, COMBAT_RATING_NAME9}, -- COMBAT_RATING_NAME9 = "Crit Rating"
		["CR_CRIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME9, PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME9},
		["CR_CRIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME9, PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME9},
		["CR_CRIT_TAKEN_MELEE"] = {"Crit Avoidance "..RATING, "Crit Avoidance "..RATING},
		["CR_CRIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance "..RATING, PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance "..RATING},
		["CR_CRIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance "..RATING, PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance "..RATING},
		["CR_RESILIENCE"] = {COMBAT_RATING_NAME15, COMBAT_RATING_NAME15}, -- COMBAT_RATING_NAME15 = "Resilience"
		["CR_HASTE_MELEE"] = {"Haste "..RATING, "Haste "..RATING}, --
		["CR_HASTE_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Haste "..RATING, PLAYERSTAT_RANGED_COMBAT.." Haste "..RATING},
		["CR_HASTE_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Haste "..RATING, PLAYERSTAT_SPELL_COMBAT.." Haste "..RATING},
		["CR_WEAPON_DAGGER"] = {"Dagger "..SKILL.." "..RATING, "Dagger "..SKILL.." "..RATING}, -- SKILL = "Skill"
		["CR_WEAPON_SWORD"] = {"Sword "..SKILL.." "..RATING, "Sword "..SKILL.." "..RATING},
		["CR_WEAPON_SWORD_2H"] = {"Sword "..SKILL.." "..RATING, "Sword "..SKILL.." "..RATING},
		["CR_WEAPON_AXE"] = {"Axe "..SKILL.." "..RATING, "Axe "..SKILL.." "..RATING},
		["CR_WEAPON_AXE_2H"] = {"Axe "..SKILL.." "..RATING, "Axe "..SKILL.." "..RATING},
		["CR_WEAPON_MACE"] = {"Mace "..SKILL.." "..RATING, "Mace "..SKILL.." "..RATING},
		["CR_WEAPON_MACE_2H"] = {"Mace "..SKILL.." "..RATING, "Mace "..SKILL.." "..RATING},
		["CR_WEAPON_GUN"] = {"Gun "..SKILL.." "..RATING, "Gun "..SKILL.." "..RATING},
		["CR_WEAPON_CROSSBOW"] = {"Crossbow "..SKILL.." "..RATING, "Crossbow "..SKILL.." "..RATING},
		["CR_WEAPON_BOW"] = {"Bow "..SKILL.." "..RATING, "Bow "..SKILL.." "..RATING},
		["CR_WEAPON_FERAL"] = {"Feral "..SKILL.." "..RATING, "Feral "..SKILL.." "..RATING},
		["CR_WEAPON_FIST"] = {"Unarmed "..SKILL.." "..RATING, "Unarmed "..SKILL.." "..RATING},

		-- Tier2 Stats
		-- Str -> AP, Block Value
		-- Agi -> AP, Crit, Dodge
		-- Sta -> Health
		-- Int -> Mana, Spell Crit
		-- Spi -> mp5oc, hp5oc
		-- Ratings -> Effect
		["DEFENSE"] = {DEFENSE, "Def"},
		["DODGE"] = {DODGE_CHANCE, DODGE},
		["PARRY"] = {PARRY_CHANCE, PARRY},
		["BLOCK"] = {BLOCK_CHANCE, BLOCK},
		["HIT_MELEE"] = {"Hit Chance", "Hit"},
		["HIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." ".."Hit Chance", PLAYERSTAT_RANGED_COMBAT.." Hit"},
		["HIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." ".."Hit Chance", PLAYERSTAT_SPELL_COMBAT.." Hit"},
		["HIT_TAKEN_MELEE"] = {"Hit Avoidance", "Hit Avoidance"},
		["HIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance", PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance"},
		["HIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance", PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance"},
		["CRIT_MELEE"] = {MELEE_CRIT_CHANCE, "Crit"}, -- MELEE_CRIT_CHANCE = "Crit Chance"
		["CRIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." "..MELEE_CRIT_CHANCE, PLAYERSTAT_RANGED_COMBAT.." Crit"},
		["CRIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." "..MELEE_CRIT_CHANCE, PLAYERSTAT_SPELL_COMBAT.." Crit"},
		["CRIT_TAKEN_MELEE"] = {"Crit Avoidance", "Crit Avoidance"},
		["CRIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance", PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance"},
		["CRIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance", PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance"},
		["HASTE_MELEE"] = {"Haste", "Haste"}, --
		["HASTE_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Haste", PLAYERSTAT_RANGED_COMBAT.." Haste"},
		["HASTE_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Haste", PLAYERSTAT_SPELL_COMBAT.." Haste"},
		["WEAPON_DAGGER"] = {"Dagger "..SKILL, "Dagger"}, -- SKILL = "Skill"
		["WEAPON_SWORD"] = {"Sword "..SKILL, "Sword"},
		["WEAPON_SWORD_2H"] = {"Sword "..SKILL, "Sword"},
		["WEAPON_AXE"] = {"Axe "..SKILL, "Axe"},
		["WEAPON_AXE_2H"] = {"Axe "..SKILL, "Axe"},
		["WEAPON_MACE"] = {"Mace "..SKILL, "Mace"},
		["WEAPON_MACE_2H"] = {"Mace "..SKILL, "Mace"},
		["WEAPON_GUN"] = {"Gun "..SKILL, "Gun"},
		["WEAPON_CROSSBOW"] = {"Crossbow "..SKILL, "Crossbow"},
		["WEAPON_BOW"] = {"Bow "..SKILL, "Bow"},
		["WEAPON_FERAL"] = {"Feral "..SKILL, "Feral"},
		["WEAPON_FIST"] = {"Unarmed "..SKILL, "Unarmed"},
		["CRIT_DAMAGE_TAKEN_MELEE"] = {"Crit Damage Taken", "Crit Damage Taken"},
		["CRIT_DAMAGE_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Crit Damage Taken", PLAYERSTAT_RANGED_COMBAT.." Crit Damage Taken"},
		["CRIT_DAMAGE_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Crit Damage Taken", PLAYERSTAT_SPELL_COMBAT.." Crit Damage Taken"},

		-- Tier3 Stats
		-- Defense -> Crit Avoidance, Hit Avoidance, Dodge, Parry, Block
		-- Weapon Skill -> Crit, Hit, Mob Dodge, Mob Parry, Mob Block
		--
	},
}

if GetLocale() == "deDE" then
elseif GetLocale() == "frFR" then
elseif GetLocale() == "koKR" then
L = {
	["Locale"] = "koKR",
	["Dual Wield"] = "쌍수 무기",
	-- By looking at the first ExcludeLen letters of a line we can exclude a lot of lines
	["ExcludeLen"] = 7,
	["Exclude"] = {
		[""] = true,
		[" \n"] = true,
		["착용 시 귀속"] = true,
		["획득 시 귀속"] = true,
		["사용 시 귀속"] = true,
		["붉은색 보석 홈"] = true,
		["노란색 보석 홈"] = true,
		["푸른색 보석 홈"] = true,
		["보라색 보석 홈"] = true,
		["퀘스트 아이템"] = true,
		["퀘스트 시작 아이템"] = true,
		["고유 "] = true, -- Unique (20)
		["최소 "] = true, -- Requires Level xx
		["\n최소"] = true, -- Requires Level xx
		["직업:"] = true, -- Classes: xx
		["종족:"] = true, -- Races: xx (vendor mounts)
		["사용 "] = true, -- Use:
		["발동 "] = true, -- Chance On Hit: 
		-- Set Bonuses
		["세트 "] = true,
		["(2) 세"] = true,
		["(2) 세"] = true,
		["(3) 세"] = true,
		["(4) 세"] = true,
		["(5) 세"] = true,
		["(6) 세"] = true,
		["(7) 세"] = true,
		["(8) 세"] = true,
		-- Equip type
		["투사체"] = true, -- Ice Threaded Arrow ID:19316
		[INVTYPE_AMMO] = true,
		[INVTYPE_HEAD] = true,
		[INVTYPE_NECK] = true,
		[INVTYPE_SHOULDER] = true,
		[INVTYPE_BODY] = true,
		[INVTYPE_CHEST] = true,
		[INVTYPE_ROBE] = true,
		[INVTYPE_WAIST] = true,
		[INVTYPE_LEGS] = true,
		[INVTYPE_FEET] = true,
		[INVTYPE_WRIST] = true,
		[INVTYPE_HAND] = true,
		[INVTYPE_FINGER] = true,
		[INVTYPE_TRINKET] = true,
		[INVTYPE_CLOAK] = true,
		[INVTYPE_WEAPON] = true,
		[INVTYPE_SHIELD] = true,
		[INVTYPE_2HWEAPON] = true,
		[INVTYPE_WEAPONMAINHAND] = true,
		[INVTYPE_WEAPONOFFHAND] = true,
		[INVTYPE_HOLDABLE] = true,
		[INVTYPE_RANGED] = true,
		[INVTYPE_THROWN] = true,
		[INVTYPE_RANGEDRIGHT] = true,
		[INVTYPE_RELIC] = true,
		[INVTYPE_TABARD] = true,
		[INVTYPE_BAG] = true,
	},
	-- Special cases that need to be dealt with before base scan
	["PreScanPatterns"] = {
		--["^Equip: Increases attack power by (%d+) in Cat"] = "FERAL_ATTACKPOWER",
		--["^Equip: Increases attack power by (%d+) when fighting Undead"] = "ATTACKPOWER_UNDEAD", -- Seal of the Dawn ID:13029
		["^(%d+)의 피해 방어$"] = "BLOCK_VALUE",
		["^방어도 (%d+)$"] = "ARMOR",
		["방어도 보강 %(%+(%d+)%)"] = "ARMOR_BUFF",
		["매 5초마다 (%d+)의 마나가 회복됩니다.$"] = "MANA_REG",
		["^.-공격력 %+?%d+ %- (%d+)$"] = "MAX_DAMAGE",
		["^%(초당 공격력 ([%d%.]+)%)$"] = "DPS",
		-- Exclude
		["^(%d+)칸"] = false, -- Set Name (0/9)
		["^[%D ]+ %((%d+)/%d+%)$"] = false, -- Set Name (0/9)
		["|cff808080"] = false, -- Gray text "  |cff808080Requires at least 2 Yellow gems|r\n  |cff808080Requires at least 1 Red gem|r"
		-- Procs
		["발동"] = false, -- [Mark of Defiance] ID:27924 -- [Staff of the Qiraji Prophets] ID:21128
		["확률로"] = false, -- [Darkmoon Card: Heroism] ID:19287
		["[Ww]hen struck in combat"] = false, -- [Essence of the Pure Flame] ID: 18815
		["성공하면"] = false, -- [Essence of the Pure Flame] ID: 18815
	},
	-- Whole Text Lookup
	-- Mainly used for enchants that doesn't have numbers in the text
	["WholeTextLookup"] = {
		["미스릴 박차"] = {["MOUNT_SPEED%"] = 4}, -- Mithril Spurs
		["최하급 탈것 속도 증가"] = {["MOUNT_SPEED%"] = 2}, -- Enchant Gloves - Riding Skill
		["착용 효과: 이동 속도가 약간 증가합니다."] = {["RUN_SPEED%"] = 8}, -- [Highlander's Plate Greaves] ID: 20048
		["이동 속도가 약간 증가합니다."] = {["RUN_SPEED%"] = 8}, -- 
		["탈것 속도 증가"] = {["RUN_SPEED%"] = 8}, -- Enchant Boots - Minor Speed
		["Equip: Allows underwater breathing."] = false, -- [Band of Icy Depths] ID: 21526
		["Allows underwater breathing"] = false, --
		["착용 효과: 무장해제에 면역이 됩니다."] = false, -- [Stronghold Gauntlets] ID: 12639
		["무장해제에 면역이 됩니다"] = false, -- 
		["성전사"] = false, -- Enchant Crusader
		["흡혈"] = false, -- Enchant Crusader
	},
	--[[
	textTable = {
		"Spell Damage +6 and Spell Hit Rating +5",
		"+3 Stamina, +4 Critical Strike Rating",
		"+26 Healing Spells & 2% Reduced Threat",
		"+3 Stamina/+4 Critical Strike Rating",
		"Socket Bonus: 2 mana per 5 sec.",
		"Equip: Increases damage and healing done by magical spells and effects by up to 150.",
		"Equip: Increases the spell critical strike rating of all party members within 30 yards by 28.",
		"Equip: Increases damage and healing done by magical spells and effects of all party members within 30 yards by up to 33.",
		"Equip: Increases healing done by magical spells and effects of all party members within 30 yards by up to 62.",
		"Equip: Increases your spell damage by up to 120 and your healing by up to 300.",
		"Equip: Restores 11 mana per 5 seconds to all party members within 30 yards.",
		"Equip: Increases healing done by spells and effects by up to 300.",
		"Equip: Increases attack power by 420 in Cat, Bear, Dire Bear, and Moonkin forms only.",
		-- "+10 Defense Rating/+10 Stamina/+15 Block Value": ZG Enchant
		-- "+26 Attack Power and +14 Critical Strike Rating": Swift Windfire Diamond ID:28556
		-- "+26 Healing Spells & 2% Reduced Threat": Bracing Earthstorm Diamond ID:25897
		-- "+6 Spell Damage, +5 Spell Crit Rating": Potent Ornate Topaz ID: 28123
		----
		-- "Critical Rating +6 and Dodge Rating +5": Assassin's Fire Opal ID:30565
		-- "Healing +11 and 2 mana per 5 sec.": Royal Tanzanite ID: 30603
	}
	--]]
	-- Single Plus Stat Check
	-- depending on locale, it may be
	-- +19 Stamina = "^%+(%d+) ([%a ]+%a)$"
	-- Stamina +19 = "^([%a ]+%a) %+(%d+)$"
	-- +19 耐力 = "^%+(%d+) (.-)$"
	--["SinglePlusStatCheck"] = "^%+(%d+) ([%a ]+%a)$",
	["SinglePlusStatCheck"] = "^(.-) %+(%d+)$",
	-- Single Equip Stat Check
	-- stat1, value, stat2 = strfind
	-- stat = stat1..stat2
	-- "^Equip: (.-) by u?p? ?t?o? ?(%d+) ?(.-)%.$"
	["SingleEquipStatCheck"] = "^착용 효과: (.-) (%d+)만큼(.-)$",
	-- Initial "+" scan
	-- Replace " and " with "@" for "+22 xxx xx and +22 xxx xxxx" type strings
	[" and "] = " and ",
	["%+(%d+) ([%a ]+%a) ?"] = "([%D ]+%D) %+(%d+) ?",
	-- DeepScan
	["Equip: "] = "착용 효과: ",
	["DeepScanSeparators"] = {
		" and ", -- "Critical Rating +6 and Dodge Rating +5": Assassin's Fire Opal ID:30565
		"/", -- "+10 Defense Rating/+10 Stamina/+15 Block Value": ZG Enchant
		" & ", -- "+26 Healing Spells & 2% Reduced Threat": Bracing Earthstorm Diamond ID:25897
		", ", -- "+6 Spell Damage, +5 Spell Crit Rating": Potent Ornate Topaz ID: 28123
		"%. ", -- "Equip: Increases attack power by 81 when fighting Undead. It also allows the acquisition of Scourgestones on behalf of the Argent Dawn.": Seal of the Dawn
	},
	--[[ OLD
	["DeepScanPatterns"] = {
		"(%a[%a ]+%a) ?%d* ?%a* by u?p? ?t?o? ?(%d+) ?a?n?d? ?", -- xxx xxx by 22 (scan first)
		"(%a[%a ]+) %+(%d+) ?a?n?d? ?", -- xxx xxx +22 (scan 2ed)
		"(%d+) ([%a ]+%a) ?a?n?d? ?", -- 22 xxx xxx (scan last)
	},
	--]]
	["DeepScanPatterns"] = {
		"^(.-) (%d+)만큼(.-)$", -- "xxx by up to 22 xxx" (scan first)
		"^(.-) 최대 (%d+)만큼(.-)$", -- "xxx by up to 22 xxx" (scan first)
		"^(.-) ?%+(%d+) ?(.-)$", -- "xxx xxx +22" or "+22 xxx xxx" or "xxx +22 xxx" (scan 2ed)
		"^(.-) ?([%d%.]+) ?(.-)$", -- 22.22 xxx xxx (scan last)
	},
	["StatIDLookup"] = {

		["은신 효과가 증가합니다"] = {"STEALTH_LEVEL"}, -- [Nightscape Boots] ID: 8197
		["무기 공격력"] = {"MELEE_DMG"}, -- Enchant
		["탈것의 속도가%만큼 증가합니다"] = {"MOUNT_SPEED%"}, -- [Highlander's Plate Greaves] ID: 20048

		["모든 능력치"] = {"STR", "AGI", "STA", "INT", "SPI",},
		["힘"] = {"STR",},
		["민첩성"] = {"AGI",},
		["체력"] = {"STA",},
		["지능"] = {"INT",},
		["정신력"] = {"SPI",},

		["비전 저항력"] = {"ARCANE_RES",},
		["화염 저항력"] = {"FIRE_RES",},
		["자연 저항력"] = {"NATURE_RES",},
		["냉기 저항력"] = {"FROST_RES",},
		["암흑 저항력"] = {"SHADOW_RES",},
		["암흑 저항력"] = {"SHADOW_RES",}, -- Demons Blood ID: 10779
		["모든 저항력"] = {"ARCANE_RES", "FIRE_RES", "FROST_RES", "NATURE_RES", "SHADOW_RES",},
		["모든 저항"] = {"ARCANE_RES", "FIRE_RES", "FROST_RES", "NATURE_RES", "SHADOW_RES",},

		["낚시"] = {"FISHING",}, -- Fishing enchant ID:846
		["낚시 숙련도"] = {"FISHING",}, -- Fishing lure
		["낚시 숙련도가 증가합니다"] = {"FISHING",}, -- Equip: Increased Fishing +20.
		["채광"] = {"MINING",}, -- Mining enchant ID:844
		["약초 채집"] = {"HERBALISM",}, -- Heabalism enchant ID:845
		["무두질"] = {"SKINNING",}, -- Skinning enchant ID:865

		["방어도"] = {"ARMOR",},
		["방어 숙련"] = {"DEFENSE",},
		["방어 숙련 증가"] = {"DEFENSE",},
		["피해 방어"] = {"BLOCK_VALUE",}, -- +22 Block Value
		["피해 방어량"] = {"BLOCK_VALUE",}, -- +22 Block Value
		["방패의 피해 방어량이 증가합니다"] = {"BLOCK_VALUE",},

		["생명력"] = {"HEALTH",},
		["HP"] = {"HEALTH",},
		["마나"] = {"MANA",},

		["전투력"] = {"ATTACKPOWER",},
		["전투력이 증가합니다"] = {"ATTACKPOWER",},
		["언데드 공격 시 전투력"] = {"ATTACKPOWER_UNDEAD",},
		["언데드 공격 시 전투력이 증가합니다"] = {"ATTACKPOWER_UNDEAD",},
		["Increases attack powerwhen fighting Undead.  It also allows the acquisition of Scourgestones on behalf of the Argent Dawn"] = {"ATTACKPOWER_UNDEAD",},
		["Increases attack powerwhen fighting Demons"] = {"ATTACKPOWER_DEMON",},
		["달빛야수 변신 상태일 때 전투력"] = {"FERAL_ATTACKPOWER",},
		["달빛야수 변신 상태일 때 전투력이 증가합니다"] = {"FERAL_ATTACKPOWER",},
		["원거리 전투력"] = {"RANGED_ATTACKPOWER",},
		["원거리 전투력이 증가합니다"] = {"RANGED_ATTACKPOWER",}, -- [High Warlord's Crossbow] ID: 18837

		["생명력 회복량"] = {"MANA_REG",},
		["Health per"] = {"HEALTH_REG",},
		["health per"] = {"HEALTH_REG",}, -- Frostwolf Insignia Rank 6 ID:17909
		["Health every"] = {"MANA_REG",},
		["health every"] = {"HEALTH_REG",}, -- [Resurgence Rod] ID:17743
		["your normal health regeneration"] = {"HEALTH_REG",}, -- Demons Blood ID: 10779
		["Restoreshealth per 5 sec"] = {"HEALTH_REG",}, -- [Onyxia Blood Talisman] ID: 18406
		["Restoreshealth every 5 sec"] = {"HEALTH_REG",}, -- [Resurgence Rod] ID:17743
		["마나 회복량"] = {"MANA_REG",},
		["Mana per"] = {"MANA_REG",},
		["mana per"] = {"MANA_REG",}, -- Resurgence Rod ID:17743 Most common
		["Mana every"] = {"MANA_REG",},
		["mana every"] = {"MANA_REG",},
		["Mana every 5 seconds"] = {"MANA_REG",}, -- [Royal Nightseye] ID: 24057
		["Mana per 5 Seconds"] = {"MANA_REG",}, -- [Royal Shadow Draenite] ID: 23109
		["Restoresmana per 5 sec"] = {"MANA_REG",}, -- [Resurgence Rod] ID:17743

		["주문 관통력"] = {"SPELLPEN",},
		["주문 관통력이 증가합니다"] = {"SPELLPEN",},
		["치유 및 주문 공격력"] = {"DMG", "HEAL",},
		["Damage and Healing Spells"] = {"DMG", "HEAL",},
		["주문 공격력 및 치유량"] = {"DMG", "HEAL",},
		["주문 공격력"] = {"DMG", "HEAL",},
		["모든 주문 및 효과의 공격력과 치유량이 증가합니다"] = {"DMG", "HEAL"},
		["주위 30미터 반경에 있는 모든 파티원의 모든 주문 및 효과의 공격력과 치유량이 증가합니다"] = {"DMG", "HEAL"}, -- Atiesh
		["주문 공격력 및 치유량"] = {"DMG", "HEAL",},
		--StatLogic:GetSum("item:22630")
		["공격력"] = {"DMG",},
		["주문 공격력이 증가합니다"] = {"DMG",}, -- Atiesh ID:22630, 22631, 22632, 22589
		["신성 피해"] = {"HOLY_DMG",},
		["비전 피해"] = {"ARCANE_DMG",},
		["화염 피해"] = {"FIRE_DMG",},
		["자연 피해"] = {"NATURE_DMG",},
		["냉기 피해"] = {"FROST_DMG",},
		["암흑 피해"] = {"SHADOW_DMG",},
		["신성 주문 공격력"] = {"HOLY_DMG",},
		["비전 주문 공격력"] = {"ARCANE_DMG",},
		["화염 주문 공격력"] = {"FIRE_DMG",},
		["자연 주문 공격력"] = {"NATURE_DMG",},
		["냉기 주문 공격력"] = {"FROST_DMG",}, -- Acrobatic Staff of Frozen Wrath ID:3185:0:0:0:0:0:1957
		["암흑 주문 공격력"] = {"SHADOW_DMG",},
		["비전 계열의 주문과 효과의 공격력이 증가합니다"] = {"ARCANE_DMG",},
		["화염 계열의 주문과 효과의 공격력이 증가합니다"] = {"FIRE_DMG",},
		["냉기 계열의 주문과 효과의 공격력이 증가합니다"] = {"FROST_DMG",}, -- Frozen Shadoweave Vest ID:21871
		["신성 계열의 주문과 효과의 공격력이 증가합니다"] = {"HOLY_DMG",},
		["자연 계열의 주문과 효과의 공격력이 증가합니다"] = {"NATURE_DMG",},
		["암흑 계열의 주문과 효과의 공격력이 증가합니다"] = {"SHADOW_DMG",}, -- Frozen Shadoweave Vest ID:21871
		["언데드에 대한 효과나 주문에 의한 피해가 증가합니다"] = {"DMG_UNDEAD"},
		["Increases damage done to Undead by magical spells and effects.  It also allows the acquisition of Scourgestones on behalf of the Argent Dawn"] = {"DMG_UNDEAD"},

		["주문 치유량"] = {"HEAL",},
		["치유량 증가"] = {"HEAL",},
		["치유량"] = {"HEAL",},
		["healing Spells"] = {"HEAL",},
		["Healing Spells"] = {"HEAL",}, -- [Royal Nightseye] ID: 24057
		["모든 주문 및 효과에 의한 치유량이 증가합니다"] = {"HEAL",},
		["주위 30미터 반경에 있는 모든 파티원의 모든 주문 및 효과에 의한 치유량이 증가합니다"] = {"HEAL",}, -- Atiesh
		["your healing"] = {"HEAL",}, -- Atiesh

		["초당 공격력"] = {"DPS",},
		["초당의 피해 추가"] = {"DPS",}, -- [Thorium Shells] ID: 15977

		--[[ Rating ID
		CR_WEAPON_SKILL = 1;
		CR_DEFENSE_SKILL = 2;
		CR_DODGE = 3;
		CR_PARRY = 4;
		CR_BLOCK = 5;
		CR_HIT_MELEE = 6;
		CR_HIT_RANGED = 7;
		CR_HIT_SPELL = 8;
		CR_CRIT_MELEE = 9;
		CR_CRIT_RANGED = 10;
		CR_CRIT_SPELL = 11;
		CR_HIT_TAKEN_MELEE = 12;
		CR_HIT_TAKEN_RANGED = 13;
		CR_HIT_TAKEN_SPELL = 14;
		CR_CRIT_TAKEN_MELEE = 15;
		CR_CRIT_TAKEN_RANGED = 16;
		CR_CRIT_TAKEN_SPELL = 17;
		CR_HASTE_MELEE = 18;
		CR_HASTE_RANGED = 19;
		CR_HASTE_SPELL = 20;
		CR_WEAPON_SKILL_MAINHAND = 21;
		CR_WEAPON_SKILL_OFFHAND = 22;
		CR_WEAPON_SKILL_RANGED = 23;
		--]]
		["방어 숙련도"] = {"CR_DEFENSE",},
		["방어 숙련도가 증가합니다"] = {"CR_DEFENSE",},
		["회피 숙련도"] = {"CR_DODGE",},
		["회피 숙련도가 증가합니다."] = {"CR_DODGE",},
		["무기 막기 숙련도"] = {"CR_PARRY",},
		["무기 막기 숙련도가 증가합니다"] = {"CR_PARRY",},
		["방패 막기 숙련도"] = {"CR_BLOCK",},
		["방패 막기 숙련도가 증가합니다"] = {"CR_BLOCK",},
		["Increases your shield block rating"] = {"CR_BLOCK",},

		["적중도"] = {"CR_HIT_MELEE",},
		["적중도가 증가합니다"] = {"CR_HIT_MELEE",}, -- ITEM_MOD_HIT_RATING
		["근접 적중도가 증가합니다"] = {"CR_HIT_MELEE",}, -- ITEM_MOD_HIT_MELEE_RATING
		["Increases your hit rating"] = {"CR_HIT_MELEE",},
		["주문 적중도"] = {"CR_HIT_SPELL",},
		["주문의 적중도"] = {"CR_HIT_SPELL",}, -- ITEM_MOD_HIT_SPELL_RATING
		["주문 적중도가 증가합니다"] = {"CR_HIT_SPELL",},
		["원거리 적중도"] = {"CR_HIT_RANGED",},
		["원거리 적중도가 증가합니다"] = {"CR_HIT_RANGED",}, -- ITEM_MOD_HIT_RANGED_RATING
		["Increases your ranged hit rating"] = {"CR_HIT_RANGED",},

		["치명타 적중도"] = {"CR_CRIT_MELEE",},
		["Critical Rating"] = {"CR_CRIT_MELEE",},
		["Critical Strike Rating"] = {"CR_CRIT_MELEE",},
		["치명타 적중도가 증가합니다"] = {"CR_CRIT_MELEE",},
		["근접 치명타 적중도가 증가합니다"] = {"CR_CRIT_MELEE",},
		["Improves critical strike rating"] = {"CR_CRIT_MELEE",},
		["주문 극대화 적중도"] = {"CR_CRIT_SPELL",},
		["주문의 극대화 적중도"] = {"CR_CRIT_SPELL",},
		["Spell Critical Rating"] = {"CR_CRIT_SPELL",},
		["Spell Crit Rating"] = {"CR_CRIT_SPELL",},
		["주문의 극대화 적중도가 증가합니다"] = {"CR_CRIT_SPELL",},
		["Increases the spell critical strike rating of all party members within 30 yards"] = {"CR_CRIT_SPELL",},
		["주문 극대화 적중도가 증가합니다"] = {"CR_CRIT_SPELL",},
		["Increases your ranged critical strike rating"] = {"CR_CRIT_RANGED",}, -- Fletcher's Gloves ID:7348

		["Improves hit avoidance rating"] = {"CR_HIT_TAKEN_MELEE"}, -- ITEM_MOD_HIT_TAKEN_RATING
		["Improves melee hit avoidance rating"] = {"CR_HIT_TAKEN_MELEE"}, -- ITEM_MOD_HIT_TAKEN_MELEE_RATING
		["Improves ranged hit avoidance rating"] = {"CR_HIT_TAKEN_RANGED"}, -- ITEM_MOD_HIT_TAKEN_RANGED_RATING
		["Improves spell hit avoidance rating"] = {"CR_HIT_TAKEN_SPELL"}, -- ITEM_MOD_HIT_TAKEN_SPELL_RATING
		["탄력도"] = {"CR_RESILIENCE",},
		["Resilience Rating"] = {"CR_RESILIENCE",},
		["탄력도가 증가합니다"] = {"CR_RESILIENCE",},
		["Improves critical avoidance rating"] = {"CR_CRIT_TAKEN_MELEE",},
		["Improves melee critical avoidance rating"] = {"CR_CRIT_TAKEN_MELEE",},
		["Improves ranged critical avoidance rating"] = {"CR_CRIT_TAKEN_RANGED",},
		["Improves spell critical avoidance rating"] = {"CR_CRIT_TAKEN_SPELL",},

		["공격 가속도"] = {"CR_HASTE_MELEE"},
		["주문 가속도"] = {"CR_HASTE_SPELL"},
		["원거리 공격 가속도"] = {"CR_HASTE_RANGED"},
		["공격 가속도가 증가합니다"] = {"CR_HASTE_MELEE"},
		["근전 공격 가속도가 증가합니다"] = {"CR_HASTE_MELEE"},
		["원거리 공격 가속도가 증가합니다"] = {"CR_HASTE_SPELL"},
		["주문 가속도가 증가합니다"] = {"CR_HASTE_RANGED"},

		["단검류 숙련도가 증가합니다"] = {"CR_WEAPON_DAGGER"},
		["한손 도검류 숙련도가 증가합니다"] = {"CR_WEAPON_SWORD"},
		["양손 도검류 숙련도가 증가합니다"] = {"CR_WEAPON_SWORD_2H"},
		["한손 도끼류 숙련도가 증가합니다"] = {"CR_WEAPON_AXE"},
		["양손 도끼류 숙련도가 증가합니다"] = {"CR_WEAPON_AXE_2H"},
		["Increases two-handed axes skill rating"] = {"CR_WEAPON_AXE_2H"},
		["한손 둔기류 숙련도가 증가합니다"] = {"CR_WEAPON_MACE"},
		["양손 둔기류 숙련도가 증가합니다"] = {"CR_WEAPON_MACE_2H"},
		["총기류 숙련도가 증가합니다"] = {"CR_WEAPON_GUN"},
		["석궁류 숙련도가 증가합니다"] = {"CR_WEAPON_CROSSBOW"},
		["활류 숙련도가 증가합니다"] = {"CR_WEAPON_BOW"},
		["야생 전투 숙련도가 증가합니다"] = {"CR_WEAPON_FERAL"},
		["장착 무기류 숙련도가 증가합니다"] = {"CR_WEAPON_FIST"}, -- Demonblood Eviscerator
		["맨손 전투 숙련도가 증가합니다"] = {"CR_WEAPON_FIST"}, -- Demonblood Eviscerator ID:27533
		-- Exclude
		["sec"] = false,
		["to"] = false,
		["칸 가방"] = false,
		["칸 화살통"] = false,
		["칸 탄환 주머니"] = false,
		["원거리 공격 속도가%만큼 증가합니다"] = false, -- AV quiver
		["원거리 무기 공격 속도가%만큼 증가합니다"] = false, -- AV quiver
	},
	-- StatID are compatible with ItemBonusLib and BonusScanner
	["StatIDToName"] = {
		["STEALTH_LEVEL"] = {"Stealth Level", "Stealth"},
		["MELEE_DMG"] = {"Melee Weapon Damage", "Weapon Damage"},
		["MOUNT_SPEED%"] = {"Mount Speed(%)", "Mount Speed(%)"},
		["RUN_SPEED%"] = {"Run Speed(%)", "Run Speed(%)"},

		["STR"] = {SPELL_STAT1_NAME, "Str"},
		["AGI"] = {SPELL_STAT2_NAME, "Agi"},
		["STA"] = {SPELL_STAT3_NAME, "Sta"},
		["INT"] = {SPELL_STAT4_NAME, "Int"},
		["SPI"] = {SPELL_STAT5_NAME, "Spi"},
		["ARMOR"] = {ARMOR.." from items", "ARMOR(Items)"},
		["ARMOR_BUFF"] = {ARMOR.." from bonus", "ARMOR(Bonus)"},

		["FIRE_RES"] = {RESISTANCE2_NAME, "FR"},
		["NATURE_RES"] = {RESISTANCE3_NAME, "NR"},
		["FROST_RES"] = {RESISTANCE4_NAME, "FrR"},
		["SHADOW_RES"] = {RESISTANCE5_NAME, "SR"},
		["ARCANE_RES"] = {RESISTANCE6_NAME, "AR"},

		["FISHING"] = {"Fishing", "Fishing"},
		["MINING"] = {"Mining", "Mining"},
		["HERBALISM"] = {"Herbalism", "Herbalism"},
		["SKINNING"] = {"Skinning", "Skinning"},

		["BLOCK_VALUE"] = {"Block Value", "Block Value"},

		["ATTACKPOWER"] = {ATTACK_POWER_TOOLTIP, "AP"},
		["RANGED_ATTACKPOWER"] = {RANGED_ATTACK_POWER, "RAP"},
		["FERAL_ATTACKPOWER"] = {ATTACK_POWER_TOOLTIP.." (Feral)", "AP(Feral)"},
		["ATTACKPOWER_UNDEAD"] = {ATTACK_POWER_TOOLTIP.." (Undead)", "AP(Undead)"},
		["ATTACKPOWER_DEMON"] = {ATTACK_POWER_TOOLTIP.." (Demon)", "AP(Demon)"},

		["HEAL"] = {"Healing", "Heal"},

		["DMG"] = {PLAYERSTAT_SPELL_COMBAT.." "..DAMAGE, PLAYERSTAT_SPELL_COMBAT.." Dmg"},
		["DMG_UNDEAD"] = {PLAYERSTAT_SPELL_COMBAT.." "..DAMAGE.." (Undead)", PLAYERSTAT_SPELL_COMBAT.." Dmg".." (Undead)"},
		["HOLY_DMG"] = {SPELL_SCHOOL1_CAP.." "..DAMAGE, SPELL_SCHOOL1_CAP.." Dmg"},
		["FIRE_DMG"] = {SPELL_SCHOOL2_CAP.." "..DAMAGE, SPELL_SCHOOL2_CAP.." Dmg"},
		["NATURE_DMG"] = {SPELL_SCHOOL3_CAP.." "..DAMAGE, SPELL_SCHOOL3_CAP.." Dmg"},
		["FROST_DMG"] = {SPELL_SCHOOL4_CAP.." "..DAMAGE, SPELL_SCHOOL4_CAP.." Dmg"},
		["SHADOW_DMG"] = {SPELL_SCHOOL5_CAP.." "..DAMAGE, SPELL_SCHOOL5_CAP.." Dmg"},
		["ARCANE_DMG"] = {SPELL_SCHOOL6_CAP.." "..DAMAGE, SPELL_SCHOOL6_CAP.." Dmg"},

		["SPELLPEN"] = {PLAYERSTAT_SPELL_COMBAT.." "..SPELL_PENETRATION, SPELL_PENETRATION},

		["HEALTH_REG"] = {HEALTH.." Regen", "HP5"},
		["MANA_REG"] = {MANA.." Regen", "MP5"},
		["HEALTH"] = {HEALTH, HP},
		["MANA"] = {MANA, MP},

		["MAX_DAMAGE"] = {"Max Damage", "Max Dmg"},
		["DPS"] = {"Damage Per Second", "DPS"},

		["CR_DEFENSE"] = {COMBAT_RATING_NAME2, COMBAT_RATING_NAME2}, -- COMBAT_RATING_NAME2 = "Defense Rating"
		["CR_DODGE"] = {COMBAT_RATING_NAME3, COMBAT_RATING_NAME3}, -- COMBAT_RATING_NAME3 = "Dodge Rating"
		["CR_PARRY"] = {COMBAT_RATING_NAME4, COMBAT_RATING_NAME4}, -- COMBAT_RATING_NAME4 = "Parry Rating"
		["CR_BLOCK"] = {COMBAT_RATING_NAME5, COMBAT_RATING_NAME5}, -- COMBAT_RATING_NAME5 = "Block Rating"
		["CR_HIT_MELEE"] = {COMBAT_RATING_NAME6, COMBAT_RATING_NAME6}, -- COMBAT_RATING_NAME6 = "Hit Rating"
		["CR_HIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME6, PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME6}, -- PLAYERSTAT_RANGED_COMBAT = "Ranged"
		["CR_HIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME6, PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME6}, -- PLAYERSTAT_SPELL_COMBAT = "Spell"
		["CR_HIT_TAKEN_MELEE"] = {"Hit Avoidance "..RATING, "Hit Avoidance "..RATING},
		["CR_HIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance "..RATING, PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance "..RATING},
		["CR_HIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance "..RATING, PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance "..RATING},
		["CR_CRIT_MELEE"] = {COMBAT_RATING_NAME9, COMBAT_RATING_NAME9}, -- COMBAT_RATING_NAME9 = "Crit Rating"
		["CR_CRIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME9, PLAYERSTAT_RANGED_COMBAT.." "..COMBAT_RATING_NAME9},
		["CR_CRIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME9, PLAYERSTAT_SPELL_COMBAT.." "..COMBAT_RATING_NAME9},
		["CR_CRIT_TAKEN_MELEE"] = {"Crit Avoidance "..RATING, "Crit Avoidance "..RATING},
		["CR_CRIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance "..RATING, PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance "..RATING},
		["CR_CRIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance "..RATING, PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance "..RATING},
		["CR_RESILIENCE"] = {COMBAT_RATING_NAME15, COMBAT_RATING_NAME15}, -- COMBAT_RATING_NAME15 = "Resilience"
		["CR_HASTE_MELEE"] = {"Haste "..RATING, "Haste "..RATING}, --
		["CR_HASTE_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Haste "..RATING, PLAYERSTAT_RANGED_COMBAT.." Haste "..RATING},
		["CR_HASTE_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Haste "..RATING, PLAYERSTAT_SPELL_COMBAT.." Haste "..RATING},
		["CR_WEAPON_DAGGER"] = {"Dagger "..SKILL.." "..RATING, "Dagger "..SKILL.." "..RATING}, -- SKILL = "Skill"
		["CR_WEAPON_SWORD"] = {"Sword "..SKILL.." "..RATING, "Sword "..SKILL.." "..RATING},
		["CR_WEAPON_SWORD_2H"] = {"Sword "..SKILL.." "..RATING, "Sword "..SKILL.." "..RATING},
		["CR_WEAPON_AXE"] = {"Axe "..SKILL.." "..RATING, "Axe "..SKILL.." "..RATING},
		["CR_WEAPON_AXE_2H"] = {"Axe "..SKILL.." "..RATING, "Axe "..SKILL.." "..RATING},
		["CR_WEAPON_MACE"] = {"Mace "..SKILL.." "..RATING, "Mace "..SKILL.." "..RATING},
		["CR_WEAPON_MACE_2H"] = {"Mace "..SKILL.." "..RATING, "Mace "..SKILL.." "..RATING},
		["CR_WEAPON_GUN"] = {"Gun "..SKILL.." "..RATING, "Gun "..SKILL.." "..RATING},
		["CR_WEAPON_CROSSBOW"] = {"Crossbow "..SKILL.." "..RATING, "Crossbow "..SKILL.." "..RATING},
		["CR_WEAPON_BOW"] = {"Bow "..SKILL.." "..RATING, "Bow "..SKILL.." "..RATING},
		["CR_WEAPON_FERAL"] = {"Feral "..SKILL.." "..RATING, "Feral "..SKILL.." "..RATING},
		["CR_WEAPON_FIST"] = {"Unarmed "..SKILL.." "..RATING, "Unarmed "..SKILL.." "..RATING},

		-- Tier2 Stats
		-- Str -> AP, Block Value
		-- Agi -> AP, Crit, Dodge
		-- Sta -> Health
		-- Int -> Mana, Spell Crit
		-- Spi -> mp5oc, hp5oc
		-- Ratings -> Effect
		["DEFENSE"] = {DEFENSE, "Def"},
		["DODGE"] = {DODGE_CHANCE, DODGE},
		["PARRY"] = {PARRY_CHANCE, PARRY},
		["BLOCK"] = {BLOCK_CHANCE, BLOCK},
		["HIT_MELEE"] = {"Hit Chance", "Hit"},
		["HIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." ".."Hit Chance", PLAYERSTAT_RANGED_COMBAT.." Hit"},
		["HIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." ".."Hit Chance", PLAYERSTAT_SPELL_COMBAT.." Hit"},
		["HIT_TAKEN_MELEE"] = {"Hit Avoidance", "Hit Avoidance"},
		["HIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance", PLAYERSTAT_RANGED_COMBAT.." Hit Avoidance"},
		["HIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance", PLAYERSTAT_SPELL_COMBAT.." Hit Avoidance"},
		["CRIT_MELEE"] = {MELEE_CRIT_CHANCE, "Crit"}, -- MELEE_CRIT_CHANCE = "Crit Chance"
		["CRIT_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." "..MELEE_CRIT_CHANCE, PLAYERSTAT_RANGED_COMBAT.." Crit"},
		["CRIT_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." "..MELEE_CRIT_CHANCE, PLAYERSTAT_SPELL_COMBAT.." Crit"},
		["CRIT_TAKEN_MELEE"] = {"Crit Avoidance", "Crit Avoidance"},
		["CRIT_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance", PLAYERSTAT_RANGED_COMBAT.." Crit Avoidance"},
		["CRIT_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance", PLAYERSTAT_SPELL_COMBAT.." Crit Avoidance"},
		["HASTE_MELEE"] = {"Haste", "Haste"}, --
		["HASTE_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Haste", PLAYERSTAT_RANGED_COMBAT.." Haste"},
		["HASTE_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Haste", PLAYERSTAT_SPELL_COMBAT.." Haste"},
		["WEAPON_DAGGER"] = {"Dagger "..SKILL, "Dagger"}, -- SKILL = "Skill"
		["WEAPON_SWORD"] = {"Sword "..SKILL, "Sword"},
		["WEAPON_SWORD_2H"] = {"Sword "..SKILL, "Sword"},
		["WEAPON_AXE"] = {"Axe "..SKILL, "Axe"},
		["WEAPON_AXE_2H"] = {"Axe "..SKILL, "Axe"},
		["WEAPON_MACE"] = {"Mace "..SKILL, "Mace"},
		["WEAPON_MACE_2H"] = {"Mace "..SKILL, "Mace"},
		["WEAPON_GUN"] = {"Gun "..SKILL, "Gun"},
		["WEAPON_CROSSBOW"] = {"Crossbow "..SKILL, "Crossbow"},
		["WEAPON_BOW"] = {"Bow "..SKILL, "Bow"},
		["WEAPON_FERAL"] = {"Feral "..SKILL, "Feral"},
		["WEAPON_FIST"] = {"Unarmed "..SKILL, "Unarmed"},
		["CRIT_DAMAGE_TAKEN_MELEE"] = {"Crit Damage Taken", "Crit Damage Taken"},
		["CRIT_DAMAGE_TAKEN_RANGED"] = {PLAYERSTAT_RANGED_COMBAT.." Crit Damage Taken", PLAYERSTAT_RANGED_COMBAT.." Crit Damage Taken"},
		["CRIT_DAMAGE_TAKEN_SPELL"] = {PLAYERSTAT_SPELL_COMBAT.." Crit Damage Taken", PLAYERSTAT_SPELL_COMBAT.." Crit Damage Taken"},

		-- Tier3 Stats
		-- Defense -> Crit Avoidance, Hit Avoidance, Dodge, Parry, Block
		-- Weapon Skill -> Crit, Hit, Mob Dodge, Mob Parry, Mob Block
		--
	},
}
elseif GetLocale() == "esES" then
end


--------------------
-- Initialization --
--------------------
local StatLogic = {}
local cache = {}
setmetatable(cache, {__mode = "kv"}) -- weak table to enable garbage collection
local tip

-- Called when a newer version is registered
local function activate(self, oldLib)
	if not oldLib then
		-- Create a custom tooltip for scanning
		tip = CreateFrame("GameTooltip", "StatLogicTooltip", nil, "GameTooltipTemplate")
		self.tip = tip
		tip:SetOwner(UIParent, "ANCHOR_NONE")
		for i = 1, 30 do
			tip[i] = _G["StatLogicTooltipTextLeft"..i]
			if not tip[i] then
				tip[i] = tip:CreateFontString()
				tip:AddFontStrings(tip[i], tip:CreateFontString())
			end
		end
	else
		tip = oldLib.tip
		self.tip = tip
	end
end


-----------
-- Tools --
-----------
local DEBUG = false

-- Localize globals
local _G = getfenv(0)
local strfind = strfind
local strsub = strsub
local gmatch = gmatch
local gsub = gsub
local pairs = pairs
local ipairs = ipairs
local type = type
local GetInventoryItemLink = GetInventoryItemLink
local IsUsableSpell = IsUsableSpell
local GetItemInfo = GetItemInfo
local UnitLevel = UnitLevel
local UnitClass = UnitClass

-- metatable for stat tables
local statTableMetatable = {
	__add = function(op1, op2)
		for k, v in pairs(op2) do
			if type(v) == "number" then
				op1[k] = (op1[k] or 0) + v
				if op1[k] == 0 then
					op1[k] = nil
				end
			end
		end
		return op1
	end,
	__sub = function(op1, op2)
		for k, v in pairs(op2) do
			if type(v) == "number" then
				op1[k] = (op1[k] or 0) - v
				if op1[k] == 0 then
					op1[k] = nil
				end
			elseif k == "itemType" then
				local i = 1
				while op1["diffItemType"..i] do
					i = i + 1
				end
				op1["diffItemType"..i] = op2.itemType
			end
		end
		return op1
	end,
}

-- Table pool borrowed from Tablet-2.0 (ckknight) --
local pool = {}

-- Delete table and return to pool
local function del(t)
	if t then
		for k in pairs(t) do
			t[k] = nil
		end
		setmetatable(t, nil)
		pool[t] = true
	end
end

local function delMulti(t)
	if t then
		for k in pairs(t) do
			if type(t[k]) == "table" then
				del(t[k])
			else
				t[k] = nil
			end
		end
		setmetatable(t, nil)
		pool[t] = true
	end
end

-- Copy table
local function copy(parent)
	local t = next(pool) or {}
	pool[t] = nil
	if parent then
		for k,v in pairs(parent) do
			t[k] = v
		end
		setmetatable(t, getmetatable(parent))
	end
	return t
end

-- New table
local function new(...)
	local t = next(pool) or {}
	pool[t] = nil

	for i = 1, select('#', ...), 2 do
		local k = select(i, ...)
		if k then
			t[k] = select(i+1, ...)
		else
			break
		end
	end
	return t
end

-- New stat table
local function newStatTable(...)
	local t = next(pool) or {}
	pool[t] = nil
	setmetatable(t, statTableMetatable)

	for i = 1, select('#', ...), 2 do
		local k = select(i, ...)
		if k then
			t[k] = select(i+1, ...)
		else
			break
		end
	end
	return t
end

StatLogic.StatTable = {}
StatLogic.StatTable.new = newStatTable
StatLogic.StatTable.del = del
StatLogic.StatTable.copy = copy
StatLogic.StatTable.metatable = statTableMetatable

-- End of Table pool --

-- copyTable
local function copyTable(to, from)
	if to then
		for k in pairs(to) do
			to[k] = nil
		end
		setmetatable(to, nil)
	else
		to = {}
	end
	for k,v in pairs(from) do
		if type(k) == "table" then
			k = copyTable({}, k)
		end
		if type(v) == "table" then
			v = copyTable({}, v)
		end
		to[k] = v
	end
	setmetatable(to, getmetatable(from))
	return to
end

local function print(text)
	if DEBUG == true then
		DEFAULT_CHAT_FRAME:AddMessage(text)
	end
end

-- SetTip("item:3185:0:0:0:0:0:1957")
function SetTip(item)
	local name, link, _, _, reqLv, _, _, _, itemType = GetItemInfo(item)
	ItemRefTooltip:ClearLines()
	ItemRefTooltip:SetHyperlink(link)
	ItemRefTooltip:Show()
end
-- End of Tools
--------------------

-- =============== --
-- Stat Conversion --
-- =============== --
----------------------------------
-- GetEffectFromRating Function --
----------------------------------
--[[ Rating ID
CR_WEAPON_SKILL = 1;
CR_DEFENSE_SKILL = 2;
CR_DODGE = 3;
CR_PARRY = 4;
CR_BLOCK = 5;
CR_HIT_MELEE = 6;
CR_HIT_RANGED = 7;
CR_HIT_SPELL = 8;
CR_CRIT_MELEE = 9;
CR_CRIT_RANGED = 10;
CR_CRIT_SPELL = 11;
CR_HIT_TAKEN_MELEE = 12;
CR_HIT_TAKEN_RANGED = 13;
CR_HIT_TAKEN_SPELL = 14;
CR_CRIT_TAKEN_MELEE = 15;
CR_CRIT_TAKEN_RANGED = 16;
CR_CRIT_TAKEN_SPELL = 17;
CR_HASTE_MELEE = 18;
CR_HASTE_RANGED = 19;
CR_HASTE_SPELL = 20;
CR_WEAPON_SKILL_MAINHAND = 21;
CR_WEAPON_SKILL_OFFHAND = 22;
CR_WEAPON_SKILL_RANGED = 23;
--]]

-- Level 60 rating base
local RatingBase = {
	[CR_WEAPON_SKILL] = 2.5,
	[CR_DEFENSE_SKILL] = 1.5,
	[CR_DODGE] = 12,
	[CR_PARRY] = 20,
	[CR_BLOCK] = 5,
	[CR_HIT_MELEE] = 10,
	[CR_HIT_RANGED] = 10,
	[CR_HIT_SPELL] = 8,
	[CR_CRIT_MELEE] = 14,
	[CR_CRIT_RANGED] = 14,
	[CR_CRIT_SPELL] = 14,
	[CR_HIT_TAKEN_MELEE] = 10, -- hit avoidance
	[CR_HIT_TAKEN_RANGED] = 10,
	[CR_HIT_TAKEN_SPELL] = 8,
	[CR_CRIT_TAKEN_MELEE] = 25, -- resilience
	[CR_CRIT_TAKEN_RANGED] = 25,
	[CR_CRIT_TAKEN_SPELL] = 25,
	[CR_HASTE_MELEE] = 20 / 3,
	[CR_HASTE_RANGED] = 20 / 3,
	[CR_HASTE_SPELL] = 20 / 3,
	[CR_WEAPON_SKILL_MAINHAND] = 2.5,
	[CR_WEAPON_SKILL_OFFHAND] = 2.5,
	[CR_WEAPON_SKILL_RANGED] = 2.5,
}

-- Formula reverse engineered by Whitetooth@Cenarius (hotdogee@巴哈姆特baha)

--[[---------------------------------
{	:GetEffectFromRating(rating, id, level)
-------------------------------------
-- Description
	Gets the effective chance from ratings
-- Args
	rating
	    number - rating value
	id
	    number - rating id (global defined in PaperDollFrame.lua)
	[level]
	    number - player level
-- Examples
	StatLogic:GetEffectFromRating(10, CR_DODGE, 70)
}
-----------------------------------]]
function StatLogic:GetEffectFromRating(rating, id, level)
	-- check for invalid input
	if type(rating) ~= "number" or id < 1 or id > 23 then return 0 end
	-- defaults to player level if not given
	level = level or UnitLevel("player")
	if level >= 60 then
		return rating/RatingBase[id]*((-3/82)*level+(131/41))
	elseif level >= 10 then
		return rating/RatingBase[id]/((1/52)*level-(8/52))
	else
		return rating/RatingBase[id]/((1/52)*10-(8/52))
	end
end


-----------------------------
-- GetCritFromAgi Function --
-----------------------------
-- Numbers reverse engineered by Whitetooth@Cenarius (hotdogee@巴哈姆特baha)
local CritPerAgi = {
	 [1] = {0.2500, 0.2174, 0.2840, 0.4476, 0.0909, 0.1663, 0.0771, 0.1500, 0.2020, },
	 [2] = {0.2381, 0.2070, 0.2834, 0.4290, 0.0909, 0.1663, 0.0771, 0.1500, 0.2020, },
	 [3] = {0.2381, 0.2070, 0.2711, 0.4118, 0.0909, 0.1583, 0.0771, 0.1429, 0.1923, },
	 [4] = {0.2273, 0.1976, 0.2530, 0.3813, 0.0865, 0.1583, 0.0735, 0.1429, 0.1923, },
	 [5] = {0.2174, 0.1976, 0.2430, 0.3677, 0.0865, 0.1511, 0.0735, 0.1429, 0.1836, },
	 [6] = {0.2083, 0.1890, 0.2337, 0.3550, 0.0865, 0.1511, 0.0735, 0.1364, 0.1836, },
	 [7] = {0.2083, 0.1890, 0.2251, 0.3321, 0.0865, 0.1511, 0.0735, 0.1364, 0.1756, },
	 [8] = {0.2000, 0.1812, 0.2171, 0.3217, 0.0826, 0.1446, 0.0735, 0.1364, 0.1756, },
	 [9] = {0.1923, 0.1812, 0.2051, 0.3120, 0.0826, 0.1446, 0.0735, 0.1304, 0.1683, },
	[10] = {0.1923, 0.1739, 0.1984, 0.2941, 0.0826, 0.1385, 0.0701, 0.1304, 0.1553, },
	[11] = {0.1852, 0.1739, 0.1848, 0.2640, 0.0826, 0.1385, 0.0701, 0.1250, 0.1496, },
	[12] = {0.1786, 0.1672, 0.1670, 0.2394, 0.0790, 0.1330, 0.0701, 0.1250, 0.1496, },
	[13] = {0.1667, 0.1553, 0.1547, 0.2145, 0.0790, 0.1330, 0.0701, 0.1250, 0.1443, },
	[14] = {0.1613, 0.1553, 0.1441, 0.1980, 0.0790, 0.1279, 0.0701, 0.1200, 0.1443, },
	[15] = {0.1563, 0.1449, 0.1330, 0.1775, 0.0790, 0.1231, 0.0671, 0.1154, 0.1346, },
	[16] = {0.1515, 0.1449, 0.1267, 0.1660, 0.0757, 0.1188, 0.0671, 0.1111, 0.1346, },
	[17] = {0.1471, 0.1403, 0.1194, 0.1560, 0.0757, 0.1188, 0.0671, 0.1111, 0.1303, },
	[18] = {0.1389, 0.1318, 0.1117, 0.1450, 0.0757, 0.1147, 0.0671, 0.1111, 0.1262, },
	[19] = {0.1351, 0.1318, 0.1060, 0.1355, 0.0727, 0.1147, 0.0671, 0.1071, 0.1262, },
	[20] = {0.1282, 0.1242, 0.0998, 0.1271, 0.0727, 0.1073, 0.0643, 0.1034, 0.1122, },
	[21] = {0.1282, 0.1208, 0.0962, 0.1197, 0.0727, 0.1073, 0.0643, 0.1000, 0.1122, },
	[22] = {0.1250, 0.1208, 0.0910, 0.1144, 0.0727, 0.1039, 0.0643, 0.1000, 0.1092, },
	[23] = {0.1190, 0.1144, 0.0872, 0.1084, 0.0699, 0.1039, 0.0643, 0.0968, 0.1063, },
	[24] = {0.1163, 0.1115, 0.0829, 0.1040, 0.0699, 0.1008, 0.0617, 0.0968, 0.1063, },
	[25] = {0.1111, 0.1087, 0.0797, 0.0980, 0.0699, 0.0978, 0.0617, 0.0909, 0.1010, },
	[26] = {0.1087, 0.1060, 0.0767, 0.0936, 0.0673, 0.0950, 0.0617, 0.0909, 0.1010, },
	[27] = {0.1064, 0.1035, 0.0734, 0.0903, 0.0673, 0.0950, 0.0617, 0.0909, 0.0985, },
	[28] = {0.1020, 0.1011, 0.0709, 0.0865, 0.0673, 0.0924, 0.0617, 0.0882, 0.0962, },
	[29] = {0.1000, 0.0988, 0.0680, 0.0830, 0.0649, 0.0924, 0.0593, 0.0882, 0.0962, },
	[30] = {0.0962, 0.0945, 0.0654, 0.0792, 0.0649, 0.0875, 0.0593, 0.0833, 0.0878, },
	[31] = {0.0943, 0.0925, 0.0637, 0.0768, 0.0649, 0.0875, 0.0593, 0.0833, 0.0859, },
	[32] = {0.0926, 0.0925, 0.0614, 0.0741, 0.0627, 0.0853, 0.0593, 0.0811, 0.0859, },
	[33] = {0.0893, 0.0887, 0.0592, 0.0715, 0.0627, 0.0831, 0.0571, 0.0811, 0.0841, },
	[34] = {0.0877, 0.0870, 0.0575, 0.0691, 0.0627, 0.0831, 0.0571, 0.0789, 0.0824, },
	[35] = {0.0847, 0.0836, 0.0556, 0.0664, 0.0606, 0.0792, 0.0571, 0.0769, 0.0808, },
	[36] = {0.0833, 0.0820, 0.0541, 0.0643, 0.0606, 0.0773, 0.0551, 0.0750, 0.0792, },
	[37] = {0.0820, 0.0820, 0.0524, 0.0628, 0.0606, 0.0773, 0.0551, 0.0732, 0.0777, },
	[38] = {0.0794, 0.0791, 0.0508, 0.0609, 0.0586, 0.0756, 0.0551, 0.0732, 0.0777, },
	[39] = {0.0781, 0.0776, 0.0493, 0.0592, 0.0586, 0.0756, 0.0551, 0.0714, 0.0762, },
	[40] = {0.0758, 0.0750, 0.0481, 0.0572, 0.0586, 0.0723, 0.0532, 0.0698, 0.0709, },
	[41] = {0.0735, 0.0737, 0.0470, 0.0556, 0.0568, 0.0707, 0.0532, 0.0682, 0.0696, },
	[42] = {0.0725, 0.0737, 0.0457, 0.0542, 0.0568, 0.0707, 0.0532, 0.0682, 0.0696, },
	[43] = {0.0704, 0.0713, 0.0444, 0.0528, 0.0551, 0.0693, 0.0532, 0.0667, 0.0685, },
	[44] = {0.0694, 0.0701, 0.0433, 0.0512, 0.0551, 0.0679, 0.0514, 0.0667, 0.0673, },
	[45] = {0.0676, 0.0679, 0.0421, 0.0497, 0.0551, 0.0665, 0.0514, 0.0638, 0.0651, },
	[46] = {0.0667, 0.0669, 0.0413, 0.0486, 0.0534, 0.0652, 0.0514, 0.0625, 0.0641, },
	[47] = {0.0649, 0.0659, 0.0402, 0.0474, 0.0534, 0.0639, 0.0498, 0.0625, 0.0641, },
	[48] = {0.0633, 0.0639, 0.0391, 0.0464, 0.0519, 0.0627, 0.0498, 0.0612, 0.0631, },
	[49] = {0.0625, 0.0630, 0.0382, 0.0454, 0.0519, 0.0627, 0.0498, 0.0600, 0.0621, },
	[50] = {0.0610, 0.0612, 0.0373, 0.0440, 0.0519, 0.0605, 0.0482, 0.0588, 0.0585, },
	[51] = {0.0595, 0.0604, 0.0366, 0.0431, 0.0505, 0.0594, 0.0482, 0.0577, 0.0577, },
	[52] = {0.0588, 0.0596, 0.0358, 0.0422, 0.0505, 0.0583, 0.0482, 0.0577, 0.0569, },
	[53] = {0.0575, 0.0580, 0.0350, 0.0412, 0.0491, 0.0583, 0.0467, 0.0566, 0.0561, },
	[54] = {0.0562, 0.0572, 0.0341, 0.0404, 0.0491, 0.0573, 0.0467, 0.0556, 0.0561, },
	[55] = {0.0549, 0.0557, 0.0334, 0.0394, 0.0478, 0.0554, 0.0467, 0.0545, 0.0546, },
	[56] = {0.0543, 0.0550, 0.0328, 0.0386, 0.0478, 0.0545, 0.0454, 0.0536, 0.0539, },
	[57] = {0.0532, 0.0544, 0.0321, 0.0378, 0.0466, 0.0536, 0.0454, 0.0526, 0.0531, },
	[58] = {0.0521, 0.0530, 0.0314, 0.0370, 0.0466, 0.0536, 0.0454, 0.0517, 0.0525, },
	[59] = {0.0510, 0.0524, 0.0307, 0.0364, 0.0454, 0.0528, 0.0441, 0.0517, 0.0518, },
	[60] = {0.0500, 0.0512, 0.0301, 0.0355, 0.0454, 0.0512, 0.0441, 0.0500, 0.0493, },
	[61] = {0.0469, 0.0491, 0.0297, 0.0334, 0.0443, 0.0496, 0.0435, 0.0484, 0.0478, },
	[62] = {0.0442, 0.0483, 0.0290, 0.0322, 0.0444, 0.0486, 0.0432, 0.0481, 0.0472, },
	[63] = {0.0418, 0.0472, 0.0284, 0.0307, 0.0441, 0.0470, 0.0424, 0.0470, 0.0456, },
	[64] = {0.0397, 0.0456, 0.0279, 0.0296, 0.0433, 0.0456, 0.0423, 0.0455, 0.0447, },
	[65] = {0.0377, 0.0446, 0.0273, 0.0286, 0.0426, 0.0449, 0.0422, 0.0448, 0.0438, },
	[66] = {0.0360, 0.0437, 0.0270, 0.0276, 0.0419, 0.0437, 0.0411, 0.0435, 0.0430, },
	[67] = {0.0344, 0.0425, 0.0264, 0.0268, 0.0414, 0.0427, 0.0412, 0.0436, 0.0424, },
	[68] = {0.0329, 0.0416, 0.0259, 0.0262, 0.0412, 0.0417, 0.0408, 0.0424, 0.0412, },
	[69] = {0.0315, 0.0408, 0.0254, 0.0256, 0.0410, 0.0408, 0.0404, 0.0414, 0.0406, },
	[70] = {0.0303, 0.0400, 0.0250, 0.0250, 0.0400, 0.0400, 0.0400, 0.0405, 0.0400, },
	[71] = {0.0297, 0.0393, 0.0246, 0.0244, 0.0390, 0.0392, 0.0396, 0.0396, 0.0394, },
	[72] = {0.0292, 0.0385, 0.0242, 0.0238, 0.0381, 0.0384, 0.0393, 0.0387, 0.0388, },
	[73] = {0.0287, 0.0378, 0.0238, 0.0233, 0.0372, 0.0377, 0.0389, 0.0379, 0.0383, },
}

local ClassNameToID = {
	["WARRIOR"] = 1,
	["PALADIN"] = 2,
	["HUNTER"] = 3,
	["ROGUE"] = 4,
	["PRIEST"] = 5,
	["SHAMAN"] = 6,
	["MAGE"] = 7,
	["WARLOCK"] = 8,
	["DRUID"] = 9,
}

--[[---------------------------------
{	:GetCritFromAgi(agi, class, level)
-------------------------------------
-- Description
	Gets the crit chance from agility
-- Args
	agi
	    number - agi value
	[class]
	    string - english class name
	    number - class id
	[level]
	    number - player level
-- Examples
	StatLogic:GetCritFromAgi(10, "WARRIOR", 70)
}
-----------------------------------]]
function StatLogic:GetCritFromAgi(agi, class, level)
	-- check for invalid input
	if type(agi) ~= "number" then return end
	-- if level is invalid input, default to player level
	if type(level) ~= "number" or level < 1 or level > 73 then
		level = UnitLevel("player")
	end
	-- if class is a class string, convert to class id
	if type(class) == "string" and ClassNameToID[string.upper(class)] ~= nil then
		class = ClassNameToID[string.upper(class)]
	-- if class is invalid input, default to player class
	elseif type(class) ~= "number" or class < 1 or class > 9 then
		local _
		_, class = UnitClass("player")
		class = ClassNameToID[class]
	end
	-- Calculate
	return agi * CritPerAgi[level][class]
end


----------------------------------
-- GetSpellCritFromInt Function --
----------------------------------
-- Numbers reverse engineered by Whitetooth@Cenarius (hotdogee@巴哈姆特baha)
local SpellCritPerInt = {
	 [1] = {0.0000, 0.0832, 0.0699, 0.0000, 0.1710, 0.1333, 0.1637, 0.1500, 0.1431, },
	 [2] = {0.0000, 0.0793, 0.0666, 0.0000, 0.1636, 0.1272, 0.1574, 0.1435, 0.1369, },
	 [3] = {0.0000, 0.0793, 0.0666, 0.0000, 0.1568, 0.1217, 0.1516, 0.1375, 0.1312, },
	 [4] = {0.0000, 0.0757, 0.0635, 0.0000, 0.1505, 0.1217, 0.1411, 0.1320, 0.1259, },
	 [5] = {0.0000, 0.0757, 0.0635, 0.0000, 0.1394, 0.1166, 0.1364, 0.1269, 0.1211, },
	 [6] = {0.0000, 0.0724, 0.0608, 0.0000, 0.1344, 0.1120, 0.1320, 0.1222, 0.1166, },
	 [7] = {0.0000, 0.0694, 0.0608, 0.0000, 0.1297, 0.1077, 0.1279, 0.1179, 0.1124, },
	 [8] = {0.0000, 0.0694, 0.0583, 0.0000, 0.1254, 0.1037, 0.1240, 0.1138, 0.1124, },
	 [9] = {0.0000, 0.0666, 0.0583, 0.0000, 0.1214, 0.1000, 0.1169, 0.1100, 0.1086, },
	[10] = {0.0000, 0.0666, 0.0559, 0.0000, 0.1140, 0.1000, 0.1137, 0.1065, 0.0984, },
	[11] = {0.0000, 0.0640, 0.0559, 0.0000, 0.1045, 0.0933, 0.1049, 0.0971, 0.0926, },
	[12] = {0.0000, 0.0616, 0.0538, 0.0000, 0.0941, 0.0875, 0.0930, 0.0892, 0.0851, },
	[13] = {0.0000, 0.0594, 0.0499, 0.0000, 0.0875, 0.0800, 0.0871, 0.0825, 0.0807, },
	[14] = {0.0000, 0.0574, 0.0499, 0.0000, 0.0784, 0.0756, 0.0731, 0.0767, 0.0750, },
	[15] = {0.0000, 0.0537, 0.0466, 0.0000, 0.0724, 0.0700, 0.0671, 0.0717, 0.0684, },
	[16] = {0.0000, 0.0537, 0.0466, 0.0000, 0.0684, 0.0666, 0.0639, 0.0688, 0.0656, },
	[17] = {0.0000, 0.0520, 0.0451, 0.0000, 0.0627, 0.0636, 0.0602, 0.0635, 0.0617, },
	[18] = {0.0000, 0.0490, 0.0424, 0.0000, 0.0597, 0.0596, 0.0568, 0.0600, 0.0594, },
	[19] = {0.0000, 0.0490, 0.0424, 0.0000, 0.0562, 0.0571, 0.0538, 0.0569, 0.0562, },
	[20] = {0.0000, 0.0462, 0.0399, 0.0000, 0.0523, 0.0538, 0.0505, 0.0541, 0.0516, },
	[21] = {0.0000, 0.0450, 0.0388, 0.0000, 0.0502, 0.0518, 0.0487, 0.0516, 0.0500, },
	[22] = {0.0000, 0.0438, 0.0388, 0.0000, 0.0470, 0.0500, 0.0460, 0.0493, 0.0477, },
	[23] = {0.0000, 0.0427, 0.0368, 0.0000, 0.0453, 0.0474, 0.0445, 0.0471, 0.0463, },
	[24] = {0.0000, 0.0416, 0.0358, 0.0000, 0.0428, 0.0459, 0.0422, 0.0446, 0.0437, },
	[25] = {0.0000, 0.0396, 0.0350, 0.0000, 0.0409, 0.0437, 0.0405, 0.0429, 0.0420, },
	[26] = {0.0000, 0.0387, 0.0341, 0.0000, 0.0392, 0.0424, 0.0390, 0.0418, 0.0409, },
	[27] = {0.0000, 0.0387, 0.0333, 0.0000, 0.0376, 0.0412, 0.0372, 0.0398, 0.0394, },
	[28] = {0.0000, 0.0370, 0.0325, 0.0000, 0.0362, 0.0394, 0.0338, 0.0384, 0.0384, },
	[29] = {0.0000, 0.0362, 0.0318, 0.0000, 0.0348, 0.0383, 0.0325, 0.0367, 0.0366, },
	[30] = {0.0000, 0.0347, 0.0304, 0.0000, 0.0333, 0.0368, 0.0312, 0.0355, 0.0346, },
	[31] = {0.0000, 0.0340, 0.0297, 0.0000, 0.0322, 0.0354, 0.0305, 0.0347, 0.0339, },
	[32] = {0.0000, 0.0333, 0.0297, 0.0000, 0.0311, 0.0346, 0.0294, 0.0333, 0.0325, },
	[33] = {0.0000, 0.0326, 0.0285, 0.0000, 0.0301, 0.0333, 0.0286, 0.0324, 0.0318, },
	[34] = {0.0000, 0.0320, 0.0280, 0.0000, 0.0289, 0.0325, 0.0278, 0.0311, 0.0309, },
	[35] = {0.0000, 0.0308, 0.0269, 0.0000, 0.0281, 0.0314, 0.0269, 0.0303, 0.0297, },
	[36] = {0.0000, 0.0303, 0.0264, 0.0000, 0.0273, 0.0304, 0.0262, 0.0295, 0.0292, },
	[37] = {0.0000, 0.0297, 0.0264, 0.0000, 0.0263, 0.0298, 0.0254, 0.0284, 0.0284, },
	[38] = {0.0000, 0.0287, 0.0254, 0.0000, 0.0256, 0.0289, 0.0248, 0.0277, 0.0276, },
	[39] = {0.0000, 0.0282, 0.0250, 0.0000, 0.0249, 0.0283, 0.0241, 0.0268, 0.0269, },
	[40] = {0.0000, 0.0273, 0.0241, 0.0000, 0.0241, 0.0272, 0.0235, 0.0262, 0.0256, },
	[41] = {0.0000, 0.0268, 0.0237, 0.0000, 0.0235, 0.0267, 0.0230, 0.0256, 0.0252, },
	[42] = {0.0000, 0.0264, 0.0237, 0.0000, 0.0228, 0.0262, 0.0215, 0.0248, 0.0244, },
	[43] = {0.0000, 0.0256, 0.0229, 0.0000, 0.0223, 0.0254, 0.0211, 0.0243, 0.0240, },
	[44] = {0.0000, 0.0256, 0.0225, 0.0000, 0.0216, 0.0248, 0.0206, 0.0236, 0.0233, },
	[45] = {0.0000, 0.0248, 0.0218, 0.0000, 0.0210, 0.0241, 0.0201, 0.0229, 0.0228, },
	[46] = {0.0000, 0.0245, 0.0215, 0.0000, 0.0206, 0.0235, 0.0197, 0.0224, 0.0223, },
	[47] = {0.0000, 0.0238, 0.0212, 0.0000, 0.0200, 0.0231, 0.0192, 0.0220, 0.0219, },
	[48] = {0.0000, 0.0231, 0.0206, 0.0000, 0.0196, 0.0226, 0.0188, 0.0214, 0.0214, },
	[49] = {0.0000, 0.0228, 0.0203, 0.0000, 0.0191, 0.0220, 0.0184, 0.0209, 0.0209, },
	[50] = {0.0000, 0.0222, 0.0197, 0.0000, 0.0186, 0.0215, 0.0179, 0.0204, 0.0202, },
	[51] = {0.0000, 0.0219, 0.0194, 0.0000, 0.0183, 0.0210, 0.0176, 0.0200, 0.0198, },
	[52] = {0.0000, 0.0216, 0.0192, 0.0000, 0.0178, 0.0207, 0.0173, 0.0195, 0.0193, },
	[53] = {0.0000, 0.0211, 0.0186, 0.0000, 0.0175, 0.0201, 0.0170, 0.0191, 0.0191, },
	[54] = {0.0000, 0.0208, 0.0184, 0.0000, 0.0171, 0.0199, 0.0166, 0.0186, 0.0186, },
	[55] = {0.0000, 0.0203, 0.0179, 0.0000, 0.0166, 0.0193, 0.0162, 0.0182, 0.0182, },
	[56] = {0.0000, 0.0201, 0.0177, 0.0000, 0.0164, 0.0190, 0.0154, 0.0179, 0.0179, },
	[57] = {0.0000, 0.0198, 0.0175, 0.0000, 0.0160, 0.0187, 0.0151, 0.0176, 0.0176, },
	[58] = {0.0000, 0.0191, 0.0170, 0.0000, 0.0157, 0.0182, 0.0149, 0.0172, 0.0173, },
	[59] = {0.0000, 0.0189, 0.0168, 0.0000, 0.0154, 0.0179, 0.0146, 0.0168, 0.0169, },
	[60] = {0.0000, 0.0185, 0.0164, 0.0000, 0.0151, 0.0175, 0.0143, 0.0165, 0.0164, },
	[61] = {0.0000, 0.0157, 0.0157, 0.0000, 0.0148, 0.0164, 0.0143, 0.0159, 0.0162, },
	[62] = {0.0000, 0.0153, 0.0154, 0.0000, 0.0145, 0.0159, 0.0143, 0.0154, 0.0157, },
	[63] = {0.0000, 0.0148, 0.0150, 0.0000, 0.0143, 0.0152, 0.0143, 0.0148, 0.0150, },
	[64] = {0.0000, 0.0143, 0.0144, 0.0000, 0.0139, 0.0147, 0.0142, 0.0143, 0.0146, },
	[65] = {0.0000, 0.0140, 0.0141, 0.0000, 0.0137, 0.0142, 0.0142, 0.0138, 0.0142, },
	[66] = {0.0000, 0.0136, 0.0137, 0.0000, 0.0134, 0.0138, 0.0138, 0.0135, 0.0137, },
	[67] = {0.0000, 0.0133, 0.0133, 0.0000, 0.0132, 0.0134, 0.0133, 0.0130, 0.0133, },
	[68] = {0.0000, 0.0131, 0.0130, 0.0000, 0.0130, 0.0131, 0.0131, 0.0127, 0.0131, },
	[69] = {0.0000, 0.0128, 0.0128, 0.0000, 0.0127, 0.0128, 0.0128, 0.0125, 0.0128, },
	[70] = {0.0000, 0.0125, 0.0125, 0.0000, 0.0125, 0.0125, 0.0125, 0.0122, 0.0125, },
	[71] = {0.0000, 0.0122, 0.0123, 0.0000, 0.0123, 0.0122, 0.0122, 0.0119, 0.0122, },
	[72] = {0.0000, 0.0120, 0.0120, 0.0000, 0.0121, 0.0120, 0.0119, 0.0116, 0.0120, },
	[73] = {0.0000, 0.0118, 0.0118, 0.0000, 0.0119, 0.0117, 0.0117, 0.0114, 0.0118, },
}

--[[---------------------------------
{	:GetSpellCritFromInt(int, class, level)
-------------------------------------
-- Description
	Gets the spell crit chance from intellect
-- Args
	int
	    number - int value
	[class]
	    string - english class name
	    number - class id
	[level]
	    number - player level
-- Examples
	StatLogic:GetSpellCritFromInt(10, "MAGE", 70)
}
-----------------------------------]]
function StatLogic:GetSpellCritFromInt(int, class, level)
	-- check for invalid input
	if type(int) ~= "number" then return end
	-- if level is invalid input, default to player level
	if type(level) ~= "number" or level < 1 or level > 73 then
		level = UnitLevel("player")
	end
	-- if class is a class string, convert to class id
	if type(class) == "string" and ClassNameToID[string.upper(class)] ~= nil then
		class = ClassNameToID[string.upper(class)]
	-- if class is invalid input, default to player class
	elseif type(class) ~= "number" or class < 1 or class > 9 then
		local _
		_, class = UnitClass("player")
		class = ClassNameToID[class]
	end
	-- Calculate
	-- defaults to player level if not given
	return int * SpellCritPerInt[level][class]
end


-- ================== --
-- Stat Summarization --
-- ================== --
--[[---------------------------------
{	:GetSum(item, table)
-------------------------------------
-- Description
	Gets a sum of stats from item
-- Args
	item
	    string - link or name of target item
	 or number - itemID of target item
	 or table - tooltip of target item
	[table]
	    table - the sum of stat values are writen to this table if provided
-- Examples
	StatLogic:GetSum(21417)
	StatLogic:GetSum("item:28040:2717")
	StatLogic:GetSum("item:19019:117") -- TF
	StatLogic:GetSum("item:3185:0:0:0:0:0:1957") -- Acrobatic Staff of Frozen Wrath ID:3185:0:0:0:0:0:1957
	StatLogic:GetSum(24396)
	SetTip("item:3185:0:0:0:0:0:1957")
}
-----------------------------------]]

function StatLogic:GetSum(item, table)
	-- Locale check
	if GetLocale() ~= L["Locale"] then return end
	local _
	-- Check item
	if (type(item) == "string") or (type(item) == "number") then
	elseif type(item) == "table" and type(item.GetItem) == "function" then
		-- Get the link
		_, item = tooltip:GetItem()
	else
		return
	end
	-- Check if item is in local cache
	local name, link, _, _, reqLv, _, _, _, itemType = GetItemInfo(item)
	if not name then return end

	-- Initialize table
	table = table or {}
	setmetatable(table, statTableMetatable)

	-- Clear table values
	for k in pairs(table) do
		table[k] = nil
	end

	-- Get data from cache if available
	if cache[link] then
		copyTable(table, cache[link])
		return table
	end

	-- Set metadata
	table.itemType = itemType
	-- Don't scan Relics because they don't have general stats
	if itemType == "INVTYPE_RELIC" then
		cache[link] = copy(table)
		return table
	end

	-- Start parsing
	tip:ClearLines() -- this is required or SetX won't work the second time its called
	tip:SetHyperlink(link)
	print(link)
	for i = 2, tip:NumLines() do
		local text = tip[i]:GetText()

		-- Trim spaces
		text = strtrim(text)
		-- Strip color codes
		if strsub(text, -2) == "|r" then
			text = strsub(text, 1, -3)
		end
		if strfind(strsub(text, 1, 10), "|c%x%x%x%x%x%x%x%x") then
			text = strsub(text, 11)
		end

		local _, g, b = tip[i]:GetTextColor()
		-- Fast Exclude --
		-- Exclude obvious strings that do not need to be checked, also exclude lines that are not white and green and normal (normal for Frozen Wrath bonus)
		if not (L.Exclude[text] or L.Exclude[strsub(text, 1, L.ExcludeLen)] or strsub(text, 1, 1) == '"' or g < 0.8 or (b < 0.99 and b > 0.1)) then
			--print(text.." = ")
			local found
			local value, stat, idTable, id
			-----------------------
			-- WholeLine Lookups --
			-----------------------
			-- Mainly used for enchants or stuff without numbers:
			-- "Mithril Spurs"
			local idTable = L.WholeTextLookup[text]
			if idTable == false then
				found = true
				print("  WholeText Exclude: "..text)
			elseif idTable then
				found = true
				for id, value in pairs(L.WholeTextLookup[text]) do
					-- sum stat
					table[id] = (table[id] or 0) + value
					print("  WholeText: "..text..", "..tostring(id).."="..tostring(value))
				end
			end
			----------------------------
			-- Single Plus Stat Check --
			----------------------------
			-- depending on locale, L.SinglePlusStatCheck may be
			-- +19 Stamina = "^%+(%d+) ([%a ]+%a)$"
			-- Stamina +19 = "^([%a ]+%a) %+(%d+)$"
			-- +19 耐力 = "^%+(%d+) (.-)$"
			if not found then
				local _, _, value, statText = strfind(text, L.SinglePlusStatCheck)
				if value then
					if tonumber(statText) then
						value, statText = statText, value
					end
					idTable = L.StatIDLookup[statText]
					if idTable == false then
						found = true
						print("  SinglePlus Exclude: "..text)
					elseif idTable then
						found = true
						local debugText = "  SinglePlus: "..text
						for _, id in ipairs(idTable) do
							--print("  '"..value.."', '"..id.."'")
							-- sum stat
							table[id] = (table[id] or 0) + tonumber(value)
							debugText = debugText..", "..tostring(id).."="..tostring(value)
						end
						print(debugText)
					else
						-- pattern match but not found in L.StatIDLookup, keep looking
					end
				end
			end
			-----------------------------
			-- Single Equip Stat Check --
			-----------------------------
			-- depending on locale, L.SingleEquipStatCheck may be
			-- "^Equip: (.-) by u?p? ?t?o? ?(%d+) ?(.-)%.$"
			if not found then
				local _, _, statText1, value, statText2 = strfind(text, L.SingleEquipStatCheck)
				if value then
					local statText = statText1..statText2
					idTable = L.StatIDLookup[statText]
					if idTable == false then
						found = true
						print("  SingleEquip Exclude: "..text)
					elseif idTable then
						found = true
						local debugText = "  SingleEquip: "..text
						for _, id in ipairs(idTable) do
							--print("  '"..value.."', '"..id.."'")
							-- sum stat
							table[id] = (table[id] or 0) + tonumber(value)
							debugText = debugText..", "..tostring(id).."="..tostring(value)
						end
						print(debugText)
					else
						-- pattern match but not found in L.StatIDLookup, keep looking
					end
				end
			end
			--[[
			-- Fast check for lines starting with "+"
			-- Works with:
			-- "+19 Stamina"
			-- ZG Enchant: +10 Defense Rating/+10 Stamina/+15 Block Value
			-- +26 Attack Power and +14 Critical Strike Rating: Swift Windfire Diamond ID:28556
			-- +26 Healing Spells & 2% Reduced Threat: Bracing Earthstorm Diamond ID:25897
			-- +6 Spell Damage, +5 Spell Crit Rating: Potent Ornate Topaz ID: 28123
			----
			-- Critical Rating +6 and Dodge Rating +5: Assassin's Fire Opal ID:30565
			-- Healing +11 and 2 mana per 5 sec.: Royal Tanzanite ID: 30603
			if (not found) and (strsub(text, 1, 1) == "+") then
				local text = gsub(text, L[" and "], "@")
				local debugText = "  +xx: "..text
				for value, stat in gmatch(text, L["%+(%d+) ([%a ]+%a) ?"]) do
					found = true
					idTable = L.StatIDLookup[stat]
					if idTable == false then
						print("  +xx Exclude: "..text)
					elseif idTable then
						for _, id in ipairs(idTable) do
							--print("  '"..value.."', '"..id.."'")
							-- sum stat
							table[id] = (table[id] or 0) + tonumber(value)
							debugText = debugText..", "..tostring(id).."="..tostring(value)
						end
					else
						print("  +xx: |cffff0000'"..stat.."'")
					end
				end
				if found then
					print(debugText)
				end
			end
			--]]
			-- PreScan for special cases, that will fit wrongly into DeepScan
			-- PreScan also has exclude patterns
			if not found then
				for pattern, id in pairs(L.PreScanPatterns) do
					found, _, value = strfind(text, pattern)
					if found then
						--found = true
						if id ~= false then
							local debugText = "  PreScan: "..text
							--print("  '"..value.."' = '"..id.."'")
							-- sum stat
							table[id] = (table[id] or 0) + tonumber(value)
							debugText = debugText..", "..tostring(id).."="..tostring(value)
							print(debugText)
						else
							print("  PreScan Exclude: "..text)
						end
						break
					end
				end
				if found then

				end
			end
			--------------
			-- DeepScan --
			--------------
			--[[
			["Equip: "] = "Equip: ",
			["DeepScanSeparators"] = {
				" and ", -- "Critical Rating +6 and Dodge Rating +5": Assassin's Fire Opal ID:30565
				"/", -- "+10 Defense Rating/+10 Stamina/+15 Block Value": ZG Enchant
				" & ", -- "+26 Healing Spells & 2% Reduced Threat": Bracing Earthstorm Diamond ID:25897
				", ", -- "+6 Spell Damage, +5 Spell Crit Rating": Potent Ornate Topaz ID: 28123
				"%. ", -- "Equip: Increases attack power by 81 when fighting Undead.  It also allows the acquisition of Scourgestones on behalf of the Argent Dawn.": Seal of the Dawn
			},
			["DeepScanPatterns"] = {
				"^(.-) by u?p? ?t?o? ?(%d+) ?(.-)$", -- "xxx by up to 22 xxx" (scan first)
				"^(.-) ?%+(%d+) ?(.-)$", -- "xxx xxx +22" or "+22 xxx xxx" or "xxx +22 xxx" (scan 2ed)
				"^(.-) ?([%d%.]+) ?(.-)$", -- 22.22 xxx xxx (scan last)
			},
			--]]
			if not found then
				-- Get a local copy
				local text = text
				-- Trim spaces
				text = strtrim(text)
				-- Strip leading "Equip: "
				text = gsub(text, L["Equip: "], "")
				-- Strip trailing "%.$"
				if strsub(text, -1) == "." then
					text = strsub(text, 1, -2)
				end
				-- Replace separators with @
				for _, sep in ipairs(L.DeepScanSeparators) do
					if strfind(text, sep) then
						text = gsub(text, sep, "@")
					end
				end
				-- Split text using @
				text = {strsplit("@", text)}
				for _, text in ipairs(text) do
					local foundWholeText = false
					-- Trim spaces
					text = strtrim(text)
					print("'"..text.."'")
					-- Whole Text Lookup
					local idTable = L.WholeTextLookup[text]
					if idTable == false then
						foundWholeText = true
						found = true
						print("  DeepScan Exclude: "..text)
					elseif idTable then
						foundWholeText = true
						found = true
						for id, value in pairs(L.WholeTextLookup[text]) do
							-- sum stat
							table[id] = (table[id] or 0) + value
							print("  DeepScan: "..text..", "..tostring(id).."="..tostring(value))
						end
					else
						-- pattern match but not found in L.WholeTextLookup, keep looking
					end
					if not foundWholeText then
						-- Pattern scan
						for _, pattern in ipairs(L.DeepScanPatterns) do
							local _, _, statText1, value, statText2 = strfind(text, pattern)
							if value then
								local statText = statText1..statText2
								idTable = L.StatIDLookup[statText]
								if idTable == false then
									found = true
									print("  DeepScan Exclude: "..text)
									break
								elseif idTable then
									found = true
									local debugText = "  DeepScan: "..text
									for _, id in ipairs(idTable) do
										--print("  '"..value.."', '"..id.."'")
										-- sum stat
										table[id] = (table[id] or 0) + tonumber(value)
										debugText = debugText..", "..tostring(id).."="..tostring(value)
									end
									print(debugText)
									break
								else
									-- pattern match but not found in L.StatIDLookup, keep looking
									print("  DeepScan Lookup Fail: |cffff0000'"..statText.."', pattern = "..pattern)
								end
							end
						end
					end
				end
			end

			if not found then
				print("  No Match: |cffff0000'"..text.."'")
				if DEBUG and RatingBuster then
					RatingBuster.db.profile.test = text
				end
			end
		else
			--print("Excluded: "..text)
		end
	end
	cache[link] = copy(table)
	return table
end

--[[---------------------------------
{	:GetDiff(item, table1, table2)
-------------------------------------
-- Description
	Gets the stat diffrence from item and equipped items
-- Args
	item
	    string - link or name of target item
	 or number - itemID of target item
	 or table - tooltip of target item
	[table1]
	    table - stat difference of item and equipped item 1 are writen to this table if provided
	[table2]
	    table - stat difference of item and equipped item 2 are writen to this table if provided
-- Examples
	StatLogic:GetDiff(21417, {}) -- Ring of Unspoken Names
	StatLogic:GetDiff(21452) -- Staff of the Ruins
}
-----------------------------------]]
local getSlotID = {
	INVTYPE_AMMO           = 0,
	INVTYPE_GUNPROJECTILE  = 0,
	INVTYPE_BOWPROJECTILE  = 0,
	INVTYPE_HEAD           = 1,
	INVTYPE_NECK           = 2,
	INVTYPE_SHOULDER       = 3,
	INVTYPE_BODY           = 4,
	INVTYPE_CHEST          = 5,
	INVTYPE_ROBE           = 5,
	INVTYPE_WAIST          = 6,
	INVTYPE_LEGS           = 7,
	INVTYPE_FEET           = 8,
	INVTYPE_WRIST          = 9,
	INVTYPE_HAND           = 10,
	INVTYPE_FINGER         = {11,12},
	INVTYPE_TRINKET        = {13,14},
	INVTYPE_CLOAK          = 15,
	INVTYPE_WEAPON         = {16,17},
	INVTYPE_2HWEAPON       = 16+17,
	INVTYPE_WEAPONMAINHAND = 16,
	INVTYPE_WEAPONOFFHAND  = 17,
	INVTYPE_SHIELD         = 17,
	INVTYPE_HOLDABLE       = 17,
	INVTYPE_RANGED         = 18,
	INVTYPE_RANGEDRIGHT    = 18,
	INVTYPE_RELIC          = 18,
	INVTYPE_GUN            = 18,
	INVTYPE_CROSSBOW       = 18,
	INVTYPE_WAND           = 18,
	INVTYPE_THROWN         = 18,
	INVTYPE_TABARD         = 19,
}

SLTest = {}

function StatLogic:GetDiff(item, diff1, diff2)
	-- Locale check
	if GetLocale() ~= L["Locale"] then return end
	-- Get item sum, results are written into diff1 table
	diff1 = self:GetSum(item, diff1)
	if not diff1 then return end
	local itemType = diff1.itemType
	-- Get equip location slot id for use in GetInventoryItemLink
	local slotID = getSlotID[itemType]
	-- Don't do bags
	if not slotID then return end

	--debug
	copyTable(SLTest, diff1)

	-- 1h weapon, check if player can dual wield, check for 2h equipped
	if itemType == "INVTYPE_WEAPON" then
		local eqItem1Sum = GetInventoryItemLink("player", 16)
		if eqItem1Sum then
			eqItem1Sum = self:GetSum(eqItem1Sum)
		else
			eqItem1Sum = newStatTable()
		end
		diff1 = diff1 - eqItem1Sum

		-- If player can Dual Wield, calculate offhand difference
		if IsUsableSpell(L["Dual Wield"]) then
			-- If 2h is equipped, copy diff1 to diff2
			if eqItem1Sum.itemType == "INVTYPE_2HWEAPON" then
				diff2 = diff2 or new()
				copyTable(diff2, diff1)
			else
				diff2 = self:GetSum(item, diff2)
				local eqItem2Sum = GetInventoryItemLink("player", 17)
				if eqItem2Sum then
					eqItem2Sum = self:GetSum(eqItem2Sum)
				else
					eqItem2Sum = newStatTable()
				end
				diff2 = diff2 - eqItem2Sum
				del(eqItem2Sum)
			end
			del(eqItem1Sum)
			return diff1, diff2
		else
			del(eqItem1Sum)
			return diff1
		end
	-- Ring or trinket
	elseif type(slotID) == "table" then
		-- Get slot link
		local eqItem1Sum = GetInventoryItemLink("player", slotID[1])
		-- Check slot link to see if slot has an item equipped
		if eqItem1Sum then
			eqItem1Sum = self:GetSum(eqItem1Sum)
		-- Else get an empty table
		else
			eqItem1Sum = newStatTable()
		end
		-- Calculate the stat difference, results are written into diff1 table
		diff1 = diff1 - eqItem1Sum
		-- Slot2
		diff2 = self:GetSum(item, diff2)
		local eqItem2Sum = GetInventoryItemLink("player", slotID[2])
		if eqItem2Sum then
			eqItem2Sum = self:GetSum(eqItem2Sum)
		else
			eqItem2Sum = newStatTable()
		end
		diff2 = diff2 - eqItem2Sum
		-- Cleanup
		del(eqItem1Sum)
		del(eqItem2Sum)
		return diff1, diff2
	-- 2h weapon, so we calculate the difference with equipped main hand and off hand
	elseif itemType == "INVTYPE_2HWEAPON" then
		local eqItem1Sum = GetInventoryItemLink("player", 16)
		if eqItem1Sum then
			eqItem1Sum = self:GetSum(eqItem1Sum)
		else
			eqItem1Sum = newStatTable()
		end
		local eqItem2Sum = GetInventoryItemLink("player", 17)
		if eqItem2Sum then
			eqItem2Sum = self:GetSum(eqItem2Sum)
		else
			eqItem2Sum = newStatTable()
		end
		diff1 = diff1 - eqItem1Sum - eqItem2Sum
		-- Cleanup
		del(eqItem1Sum)
		del(eqItem2Sum)
		return diff1
	-- Off hand slot, check if we have 2h equipped
	elseif slotID == 17 then
		local eqItem1Sum = GetInventoryItemLink("player", 16)
		if eqItem1Sum then
			eqItem1Sum = self:GetSum(eqItem1Sum)
		else
			eqItem1Sum = newStatTable()
		end
		-- If 2h is equipped
		if eqItem1Sum.itemType == "INVTYPE_2HWEAPON" then
			diff1 = diff1 - eqItem1Sum
		else
			local eqItem1Sum = GetInventoryItemLink("player", 17)
			if eqItem1Sum then
				eqItem1Sum = self:GetSum(eqItem1Sum)
			else
				eqItem1Sum = newStatTable()
			end
			diff1 = diff1 - eqItem1Sum
			del(eqItem1Sum)
		end
		del(eqItem1Sum)
		return diff1
	-- Single slot item
	else
		local eqItem1Sum = GetInventoryItemLink("player", slotID)
		if eqItem1Sum then
			eqItem1Sum = self:GetSum(eqItem1Sum)
		else
			eqItem1Sum = newStatTable()
		end
		diff1 = diff1 - eqItem1Sum
		-- Cleanup
		del(eqItem1Sum)
		return diff1
	end
end


-- StatLogic:Bench(1000)
---------
-- self:GetSum(link, table)
-- 1000 times: 0.82 sec without cache
-- 1000 times: 0.04 sec with cache
---------
-- ItemBonusLib:ScanItemLink(link)
-- 1000 times: 1.58 sec
---------
function StatLogic:Bench(k)
	local t1 = GetTime()
	local link = GetInventoryItemLink("player", 12)
	local table = {}
	for i = 1, k, 1 do
		---------------------------------------------------------------------------
		--self:SplitDoJoin("+24 Agility/+4 Stamina, +4 Dodge and +4 Spell Crit/+5 Spirit", {"/", " and ", ","})
		---------------------------------------------------------------------------
		self:GetSum(link)
		--ItemBonusLib:ScanItemLink(link)
		---------------------------------------------------------------------------
		--ItemRefTooltip:SetScript("OnTooltipSetItem", function(frame, ...) RatingBuster:Print("OnTooltipSetItem") end)
		---------------------------------------------------------------------------
	end
	return GetTime() - t1
end


function StatLogic:PatternTest()
	patternTable = {
		"(%a[%a ]+%a) ?%d* ?%a* by u?p? ?t?o? ?(%d+) ?a?n?d? ?", -- xxx xxx by 22 (scan first)
		"(%a[%a ]+) %+(%d+) ?a?n?d? ?", -- xxx xxx +22 (scan 2ed)
		"(%d+) ([%a ]+) ?a?n?d? ?", -- 22 xxx xxx (scan last)
	}
	textTable = {
		"Spell Damage +6 and Spell Hit Rating +5",
		"+3 Stamina, +4 Critical Strike Rating",
		"+26 Healing Spells & 2% Reduced Threat",
		"+3 Stamina/+4 Critical Strike Rating",
		"Socket Bonus: 2 mana per 5 sec.",
		"Equip: Increases damage and healing done by magical spells and effects by up to 150.",
		"Equip: Increases the spell critical strike rating of all party members within 30 yards by 28.",
		"Equip: Increases damage and healing done by magical spells and effects of all party members within 30 yards by up to 33.",
		"Equip: Increases healing done by magical spells and effects of all party members within 30 yards by up to 62.",
		"Equip: Increases your spell damage by up to 120 and your healing by up to 300.",
		"Equip: Restores 11 mana per 5 seconds to all party members within 30 yards.",
		"Equip: Increases healing done by spells and effects by up to 300.",
		"Equip: Increases attack power by 420 in Cat, Bear, Dire Bear, and Moonkin forms only.",
	}
	for _, text in ipairs(textTable) do
		DEFAULT_CHAT_FRAME:AddMessage(text.." = ")
		for _, pattern in ipairs(patternTable) do
			local found
			for k, v in string.gmatch(text, pattern) do
				found = true
				DEFAULT_CHAT_FRAME:AddMessage("  '"..k.."', '"..v.."'")
			end
			if found then
				DEFAULT_CHAT_FRAME:AddMessage("  using: "..pattern)
				DEFAULT_CHAT_FRAME:AddMessage("----------------------------")
				break
			end
		end
	end
end

AceLibrary:Register(StatLogic, MAJOR_VERSION, MINOR_VERSION, activate)

StatLogic = nil
